@@ -2,11 +2,11 @@ from __future__ import absolute_import, division, print_function
 
 from ..utils import ignoring
 from .core import (Array, stack, concatenate, take, tensordot, transpose,
-        from_array, from_imperative, choose, where, coarsen, insert,
-        broadcast_to, ravel, reshape, fromfunction, unique, store, squeeze,
-        topk, bincount, digitize, histogram, map_blocks, atop, to_hdf5, dot, cov, array,
+        from_array, choose, where, coarsen, insert, broadcast_to, ravel,
+        reshape, fromfunction, unique, store, squeeze, topk, bincount,
+        digitize, histogram, map_blocks, atop, to_hdf5, dot, cov, array,
         dstack, vstack, hstack, to_npy_stack, from_npy_stack, compress,
-        from_delayed)
+        from_delayed, swapaxes)
 from .core import (logaddexp, logaddexp2, conj, exp, log, log2, log10, log1p,
         expm1, sqrt, square, sin, cos, tan, arcsin, arccos, arctan, arctan2,
         hypot, sinh, cosh, tanh, arcsinh, arccosh, arctanh, deg2rad, rad2deg,
