@@ -18,15 +18,15 @@ from .core import concatenate3, Array, normalize_chunks
 
 
 def cumdims_label(chunks, const):
-    """ Interal utility for cumulative sum with label.
+    """ Internal utility for cumulative sum with label.
 
     >>> cumdims_label(((5, 3, 3), (2, 2, 1)), 'n')  # doctest: +NORMALIZE_WHITESPACE
     [(('n', 0), ('n', 5), ('n', 8), ('n', 11)),
      (('n', 0), ('n', 2), ('n', 4), ('n', 5))]
     """
     return [tuple(zip((const,) * (1 + len(bds)),
-                      list(accumulate(add, (0,) + bds))))
-              for bds in chunks ]
+                  list(accumulate(add, (0,) + bds))))
+            for bds in chunks]
 
 
 def _breakpoints(cumold, cumnew):
@@ -76,7 +76,7 @@ def _intersect_1d(breaks):
     ret = [[]]
     for idx in range(1, len(breaks)):
         bi = breaks[idx]
-        lastbi = breaks[idx -1]
+        lastbi = breaks[idx - 1]
         if 'n' in lastbi[0] and bi[1]:
             ret.append([])
         if 'o' in lastbi[0]:
@@ -201,32 +201,40 @@ def rechunk(x, chunks):
     chunks = normalize_chunks(chunks, x.shape)
     if chunks == x.chunks:
         return x
-    if not len(chunks) == x.ndim or tuple(map(sum, chunks)) != x.shape:
+    ndim = x.ndim
+    if not len(chunks) == ndim or tuple(map(sum, chunks)) != x.shape:
         raise ValueError("Provided chunks are not consistent with shape")
 
     crossed = intersect_chunks(x.chunks, chunks)
     x2 = dict()
-    temp_name = 'rechunk-' + tokenize(x, chunks)
+    intermediates = dict()
+    token = tokenize(x, chunks)
+    temp_name = 'rechunk-merge-' + token
     new_index = tuple(product(*(tuple(range(len(n))) for n in chunks)))
     for flat_idx, cross1 in enumerate(crossed):
         new_idx = new_index[flat_idx]
         key = (temp_name,) + new_idx
         cr2 = iter(cross1)
-        old_blocks = tuple(tuple(ind  for ind,_ in cr) for cr in cross1)
-        subdims = tuple(len(set(ss[i] for ss in old_blocks)) for i in range(x.ndim))
-        rec_cat_arg =np.empty(subdims).tolist()
-        inds_in_block = product(*(range(s) for s in subdims))
+        old_blocks = [[ind for ind, _ in cr] for cr in cross1]
+        subdims = [len(set([ss[i] for ss in old_blocks])) for i in range(ndim)]
+        rec_cat_arg = np.empty(subdims).tolist()
+        inds_in_block = product(*[range(s) for s in subdims])
         for old_block in old_blocks:
             ind_slics = next(cr2)
-            old_inds = tuple(tuple(s[0] for s in ind_slics) for i in range(x.ndim))
+            old_inds = [[s[0] for s in ind_slics] for i in range(ndim)]
             # list of nd slices
-            slic = tuple(tuple(s[1] for s in ind_slics)  for i in range(x.ndim))
+            slic = [[s[1] for s in ind_slics] for i in range(ndim)]
             ind_in_blk = next(inds_in_block)
             temp = rec_cat_arg
-            for i in range(x.ndim -1):
+            for i in range(ndim - 1):
                 temp = getitem(temp, ind_in_blk[i])
             for ind, slc in zip(old_inds, slic):
-                temp[ind_in_blk[-1]] = (getitem, (x.name,) + ind, slc)
+                name = (('rechunk-split-' + token,)
+                        + tuple(ind)
+                        + sum([(s.start, s.stop) for s in slc], ()))
+                intermediates[name] = (getitem, (x.name,) + tuple(ind),
+                                                 tuple(slc))
+                temp[ind_in_blk[-1]] = name
         x2[key] = (concatenate3, rec_cat_arg)
-    x2 = merge(x.dask, x2)
+    x2 = merge(x.dask, x2, intermediates)
     return Array(x2, temp_name, chunks, dtype=x.dtype)
