@@ -139,7 +139,6 @@ def slice_array(out_name, in_name, blockdims, index):
     return dsk_out, bd_out
 
 
-
 def slice_with_newaxes(out_name, in_name, blockdims, index):
     """
     Handle indexing with Nones
@@ -182,7 +181,8 @@ def slice_wrap_lists(out_name, in_name, blockdims, index):
     """
     shape = tuple(map(sum, blockdims))
     assert all(isinstance(i, (slice, list, int, long)) for i in index)
-    assert len(blockdims) == len(index)
+    if not len(blockdims) == len(index):
+        raise IndexError("Too many indices for array")
     for bd, i in zip(blockdims, index):
         check_index(i, sum(bd))
 
@@ -196,9 +196,8 @@ def slice_wrap_lists(out_name, in_name, blockdims, index):
 
     # Replace all lists with full slices  [3, 1, 0] -> slice(None, None, None)
     index_without_list = tuple(slice(None, None, None)
-                                    if isinstance(i, list)
-                                    else i
-                                    for i in index2)
+                               if isinstance(i, list) else i
+                               for i in index2)
 
     # No lists, hooray! just use slice_slices_and_integers
     if index2 == index_without_list:
@@ -253,8 +252,8 @@ def slice_slices_and_integers(out_name, in_name, blockdims, index):
     # (out_name, 0, 0, 0), (out_name, 0, 0, 1), (out_name, 0, 1, 0), ...
     out_names = list(product([out_name],
                              *[range(len(d))[::-1] if i.step and i.step < 0 else range(len(d))
-                                 for d, i in zip(block_slices, index)
-                                 if not isinstance(i, (int, long))]))
+                               for d, i in zip(block_slices, index)
+                               if not isinstance(i, (int, long))]))
 
     all_slices = list(product(*[pluck(1, s) for s in sorted_block_slices]))
 
@@ -402,20 +401,13 @@ def partition_by_size(sizes, seq):
     >>> partition_by_size([10, 20, 10], [1, 5, 9, 12, 29, 35])
     [[1, 5, 9], [2, 19], [5]]
     """
-    seq = list(seq)
-    pretotal = 0
-    total = 0
-    i = 0
-    result = list()
-    for s in sizes:
-        total += s
-        L = list()
-        while i < len(seq) and seq[i] < total:
-            L.append(seq[i] - pretotal)
-            i += 1
-        result.append(L)
-        pretotal += s
-    return result
+    seq = np.array(seq)
+    right = np.cumsum(sizes)
+    locations = np.searchsorted(seq, right)
+    locations = [0] + locations.tolist()
+    left = [0] + right.tolist()
+    return [(seq[locations[i]:locations[i + 1]] - left[i]).tolist()
+            for i in range(len(locations) - 1)]
 
 
 def issorted(seq):
@@ -459,7 +451,6 @@ def take_sorted(outname, inname, blockdims, index, axis=0):
     --------
     take - calls this function
     """
-    n = len(blockdims)
     sizes = blockdims[axis]  # the blocksizes on the axis that we care about
 
     index_lists = partition_by_size(sizes, sorted(index))
@@ -474,7 +465,7 @@ def take_sorted(outname, inname, blockdims, index, axis=0):
 
     outdims = list(dims)
     outdims[axis] = where_index
-    slices = [[colon]*len(bd) for bd in blockdims]
+    slices = [[colon] * len(bd) for bd in blockdims]
     slices[axis] = index_lists
     slices = list(product(*slices))
     inkeys = list(product([inname], *outdims))
@@ -520,21 +511,20 @@ def take(outname, inname, blockdims, index, axis=0):
     index_lists = partition_by_size(sizes, sorted(index))
 
     dims = [[0] if axis == i else list(range(len(bd)))
-                for i, bd in enumerate(blockdims)]
+            for i, bd in enumerate(blockdims)]
     keys = list(product([outname], *dims))
 
     rev_index = list(map(sorted(index).index, index))
-    vals = [(getitem, (np.concatenate,
-                  (list, [(getitem, ((inname,) + d[:axis] + (i,) + d[axis+1:]),
-                                   ((colon,)*axis + (IL,) + (colon,)*(n-axis-1)))
-                                for i, IL in enumerate(index_lists)
-                                if IL]),
-                        axis),
-                     ((colon,)*axis + (rev_index,) + (colon,)*(n-axis-1)))
+    vals = [(getitem,
+             (np.concatenate,
+              (list, [(getitem, ((inname, ) + d[:axis] + (i, ) + d[axis + 1:]),
+                      ((colon, ) * axis + (IL, ) + (colon, ) * (n - axis - 1)))
+                      for i, IL in enumerate(index_lists) if IL]), axis),
+             ((colon, ) * axis + (rev_index, ) + (colon, ) * (n - axis - 1)))
             for d in product(*dims)]
 
     blockdims2 = list(blockdims)
-    blockdims2[axis] = (len(index),)
+    blockdims2[axis] = (len(index), )
 
     return tuple(blockdims2), dict(zip(keys, vals))
 
@@ -562,7 +552,7 @@ def posify_index(shape, ind):
         else:
             return ind
     if isinstance(ind, list):
-        return [posify_index(shape, i) for i in ind]
+        return [i + shape if i < 0 else i for i in ind]
     return ind
 
 
@@ -599,7 +589,7 @@ def new_blockdim(dim_shape, lengths, index):
     assert not isinstance(index, (int, long))
     pairs = sorted(_slice_1d(dim_shape, lengths, index).items(), key=first)
     slices = [slice(0, lengths[i], 1) if slc == slice(None, None, None) else slc
-                for i, slc in pairs]
+              for i, slc in pairs]
     if isinstance(index, slice) and index.step and index.step < 0:
         slices = slices[::-1]
     return [int(ceil((1. * slc.stop - slc.start) / slc.step)) for slc in slices]
@@ -617,9 +607,8 @@ def replace_ellipsis(n, index):
         return index
     else:
         loc = isellipsis[0]
-    return (index[:loc]
-          + (slice(None, None, None),) * (n - len(index) + 1)
-          + index[loc+1:])
+    return (index[:loc] + (slice(None, None, None),) * (n - len(index) + 1) +
+            index[loc + 1:])
 
 
 def check_index(ind, dimension):
@@ -648,21 +637,21 @@ def check_index(ind, dimension):
     >>> check_index([6, 3], 5)
     Traceback (most recent call last):
     ...
-    IndexError: Index is not smaller than dimension 6 >= 5
+    IndexError: Index out of bounds 5
 
     >>> check_index(slice(0, 3), 5)
     """
     if isinstance(ind, list):
-        for i in ind:
-            check_index(i, dimension)
+        x = np.array(ind)
+        if (x >= dimension).any() or (x <= -dimension).any():
+            raise IndexError("Index out of bounds %s" % dimension)
     elif isinstance(ind, slice):
         return
 
     elif ind >= dimension:
-        raise IndexError("Index is not smaller than dimension %d >= %d"
-                % (ind, dimension))
+        raise IndexError("Index is not smaller than dimension %d >= %d" %
+                         (ind, dimension))
 
     elif ind <= -dimension:
-        raise IndexError(
-                "Negative index is not greater than negative dimension %d <= -%d"
-                % (ind, dimension))
+        msg = "Negative index is not greater than negative dimension %d <= -%d"
+        raise IndexError(msg % (ind, dimension))
