@@ -2,22 +2,11 @@ import pytest
 pytest.importorskip('numpy')
 
 from dask.utils import skip
+from dask.array.utils import assert_eq
 import dask.array as da
-import dask
 import numpy as np
 
 
-def eq(a, b):
-    if isinstance(a, da.Array):
-        a = a.compute(get=dask.get)
-    if isinstance(b, da.Array):
-        b = b.compute(get=dask.get)
-    c = a == b
-    if isinstance(c, np.ndarray):
-        c = c.all()
-    return c
-
-
 def same_keys(a, b):
     def key(k):
         if isinstance(k, str):
@@ -29,11 +18,14 @@ def same_keys(a, b):
 
 def test_percentile():
     d = da.ones((16,), chunks=(4,))
-    assert eq(da.percentile(d, [0, 50, 100]), [1, 1, 1])
+    assert_eq(da.percentile(d, [0, 50, 100]),
+              np.array([1, 1, 1], dtype=d.dtype))
 
     x = np.array([0, 0, 5, 5, 5, 5, 20, 20])
     d = da.from_array(x, chunks=(3,))
-    assert eq(da.percentile(d, [0, 50, 100]), [0, 5, 20])
+    result = da.percentile(d, [0, 50, 100])
+    assert_eq(da.percentile(d, [0, 50, 100]),
+              np.array([0, 5, 20], dtype=result.dtype))
     assert same_keys(da.percentile(d, [0, 50, 100]),
                     da.percentile(d, [0, 50, 100]))
     assert not same_keys(da.percentile(d, [0, 50, 100]),
@@ -41,7 +33,8 @@ def test_percentile():
 
     x = np.array(['a', 'a', 'd', 'd', 'd', 'e'])
     d = da.from_array(x, chunks=(3,))
-    assert eq(da.percentile(d, [0, 50, 100]), ['a', 'd', 'e'])
+    assert_eq(da.percentile(d, [0, 50, 100]),
+              np.array(['a', 'd', 'e'], dtype=x.dtype))
 
 
 @skip
@@ -66,4 +59,4 @@ def test_percentile_with_categoricals():
 
 def test_percentiles_with_empty_arrays():
     x = da.ones(10, chunks=((5, 0, 5),))
-    assert da.percentile(x, [10, 50, 90]).compute().tolist() == [1, 1, 1]
+    assert_eq(da.percentile(x, [10, 50, 90]), np.array([1, 1, 1], dtype=x.dtype))
