@@ -1,29 +1,16 @@
 import pytest
 pytest.importorskip('numpy')
 
-import dask
 from dask.compatibility import skip
 import dask.array as da
-from dask.array import Array
 from dask.array.slicing import (slice_array, _slice_1d, take, new_blockdim,
         sanitize_index)
+from dask.array.utils import assert_eq
 from operator import getitem
 import numpy as np
 from toolz import merge
 
 
-def eq(a, b):
-    if isinstance(a, da.Array):
-        a = a.compute(get=dask.get)
-    if isinstance(b, da.Array):
-        b = b.compute(get=dask.get)
-
-    c = a == b
-    if isinstance(c, np.ndarray):
-        c = c.all()
-    return c
-
-
 def same_keys(a, b):
     def key(k):
         if isinstance(k, str):
@@ -35,117 +22,117 @@ def same_keys(a, b):
 
 def test_slice_1d():
     expected = {0: slice(10, 25, 1), 1: slice(None, None, None), 2: slice(0, 1, 1)}
-    result = _slice_1d(100, [25]*4, slice(10, 51, None))
+    result = _slice_1d(100, [25] * 4, slice(10, 51, None))
     assert expected == result
 
-    #x[100:12:-3]
+    # x[100:12:-3]
     expected = {0: slice(-2, -8, -3),
                 1: slice(-1, -21, -3),
                 2: slice(-3, -21, -3),
                 3: slice(-2, -21, -3),
                 4: slice(-1, -21, -3)}
-    result = _slice_1d(100, [20]*5, slice(100, 12, -3))
+    result = _slice_1d(100, [20] * 5, slice(100, 12, -3))
     assert expected == result
 
-    #x[102::-3]
+    # x[102::-3]
     expected = {0: slice(-2, -21, -3),
                 1: slice(-1, -21, -3),
                 2: slice(-3, -21, -3),
                 3: slice(-2, -21, -3),
                 4: slice(-1, -21, -3)}
-    result = _slice_1d(100, [20]*5, slice(102, None, -3))
+    result = _slice_1d(100, [20] * 5, slice(102, None, -3))
     assert expected == result
 
-    #x[::-4]
+    # x[::-4]
     expected = {0: slice(-1, -21, -4),
                 1: slice(-1, -21, -4),
                 2: slice(-1, -21, -4),
                 3: slice(-1, -21, -4),
                 4: slice(-1, -21, -4)}
-    result = _slice_1d(100, [20]*5, slice(None, None, -4))
+    result = _slice_1d(100, [20] * 5, slice(None, None, -4))
     assert expected == result
 
-    #x[::-7]
+    # x[::-7]
     expected = {0: slice(-5, -21, -7),
                 1: slice(-4, -21, -7),
                 2: slice(-3, -21, -7),
                 3: slice(-2, -21, -7),
                 4: slice(-1, -21, -7)}
-    result = _slice_1d(100, [20]*5, slice(None, None, -7))
+    result = _slice_1d(100, [20] * 5, slice(None, None, -7))
     assert expected == result
 
-    #x=range(115)
-    #x[::-7]
+    # x=range(115)
+    # x[::-7]
     expected = {0: slice(-7, -24, -7),
                 1: slice(-2, -24, -7),
                 2: slice(-4, -24, -7),
                 3: slice(-6, -24, -7),
                 4: slice(-1, -24, -7)}
-    result = _slice_1d(115, [23]*5, slice(None, None, -7))
+    result = _slice_1d(115, [23] * 5, slice(None, None, -7))
     assert expected == result
 
-    #x[79::-3]
+    # x[79::-3]
     expected = {0: slice(-1, -21, -3),
                 1: slice(-3, -21, -3),
                 2: slice(-2, -21, -3),
                 3: slice(-1, -21, -3)}
-    result = _slice_1d(100, [20]*5, slice(79, None, -3))
+    result = _slice_1d(100, [20] * 5, slice(79, None, -3))
     assert expected == result
 
-    #x[-1:-8:-1]
+    # x[-1:-8:-1]
     expected = {4: slice(-1, -8, -1)}
     result = _slice_1d(100, [20, 20, 20, 20, 20], slice(-1, 92, -1))
     assert expected == result
 
-    #x[20:0:-1]
+    # x[20:0:-1]
     expected = {0: slice(-1, -20, -1),
                 1: slice(-20, -21, -1)}
     result = _slice_1d(100, [20, 20, 20, 20, 20], slice(20, 0, -1))
     assert expected == result
 
-    #x[:0]
+    # x[:0]
     expected = {}
     result = _slice_1d(100, [20, 20, 20, 20, 20], slice(0))
     assert result
 
-    #x=range(99)
+    # x=range(99)
     expected = {0: slice(-3, -21, -3),
                 1: slice(-2, -21, -3),
                 2: slice(-1, -21, -3),
                 3: slice(-2, -20, -3),
                 4: slice(-1, -21, -3)}
-    #This array has non-uniformly sized blocks
+    # This array has non-uniformly sized blocks
     result = _slice_1d(99, [20, 20, 20, 19, 20], slice(100, None, -3))
     assert expected == result
 
-    #x=range(104)
-    #x[::-3]
+    # x=range(104)
+    # x[::-3]
     expected = {0: slice(-1, -21, -3),
                 1: slice(-3, -24, -3),
                 2: slice(-3, -28, -3),
                 3: slice(-1, -14, -3),
                 4: slice(-1, -22, -3)}
-    #This array has non-uniformly sized blocks
+    # This array has non-uniformly sized blocks
     result = _slice_1d(104, [20, 23, 27, 13, 21], slice(None, None, -3))
     assert expected == result
 
-    #x=range(104)
-    #x[:27:-3]
+    # x=range(104)
+    # x[:27:-3]
     expected = {1: slice(-3, -16, -3),
                 2: slice(-3, -28, -3),
                 3: slice(-1, -14, -3),
                 4: slice(-1, -22, -3)}
-    #This array has non-uniformly sized blocks
+    # This array has non-uniformly sized blocks
     result = _slice_1d(104, [20, 23, 27, 13, 21], slice(None, 27, -3))
     assert expected == result
 
-    #x=range(104)
-    #x[100:27:-3]
+    # x=range(104)
+    # x[100:27:-3]
     expected = {1: slice(-3, -16, -3),
                 2: slice(-3, -28, -3),
                 3: slice(-1, -14, -3),
                 4: slice(-4, -22, -3)}
-    #This array has non-uniformly sized blocks
+    # This array has non-uniformly sized blocks
     result = _slice_1d(104, [20, 23, 27, 13, 21], slice(100, 27, -3))
     assert expected == result
 
@@ -161,7 +148,7 @@ def test_slice_array_1d():
                 ('y', 1): (getitem, ('x', 1), (slice(1, 25, 2),)),
                 ('y', 2): (getitem, ('x', 2), (slice(0, 25, 2),)),
                 ('y', 3): (getitem, ('x', 3), (slice(1, 25, 2),))}
-    result, chunks = slice_array('y', 'x', [[25]*4], [slice(24,None,2)])
+    result, chunks = slice_array('y', 'x', [[25] * 4], [slice(24, None, 2)])
 
     assert expected == result
 
@@ -170,7 +157,7 @@ def test_slice_array_1d():
                 ('y', 1): (getitem, ('x', 2), (slice(0, 25, 2),)),
                 ('y', 2): (getitem, ('x', 3), (slice(1, 25, 2),))}
 
-    result, chunks = slice_array('y', 'x', [[25]*4], [slice(26,None,2)])
+    result, chunks = slice_array('y', 'x', [[25] * 4], [slice(26, None, 2)])
     assert expected == result
 
     #x[24::2]
@@ -178,7 +165,7 @@ def test_slice_array_1d():
                 ('y', 1): (getitem, ('x', 1), (slice(1, 25, 2),)),
                 ('y', 2): (getitem, ('x', 2), (slice(0, 25, 2),)),
                 ('y', 3): (getitem, ('x', 3), (slice(1, 25, 2),))}
-    result, chunks = slice_array('y', 'x', [(25,)*4], (slice(24,None,2),))
+    result, chunks = slice_array('y', 'x', [(25, ) * 4], (slice(24, None, 2), ))
 
     assert expected == result
 
@@ -187,7 +174,7 @@ def test_slice_array_1d():
                 ('y', 1): (getitem, ('x', 2), (slice(0, 25, 2),)),
                 ('y', 2): (getitem, ('x', 3), (slice(1, 25, 2),))}
 
-    result, chunks = slice_array('y', 'x', [(25,)*4], (slice(26,None,2),))
+    result, chunks = slice_array('y', 'x', [(25, ) * 4], (slice(26, None, 2), ))
     assert expected == result
 
 
@@ -204,7 +191,7 @@ def test_slice_array_2d():
                                (slice(13, 20, 2), slice(None, None, None)))}
 
     result, chunks = slice_array('y', 'x', [[20], [20, 20, 5]],
-                        [slice(13,None,2), slice(10, None, 1)])
+                        [slice(13, None, 2), slice(10, None, 1)])
 
     assert expected == result
 
@@ -228,16 +215,17 @@ def test_slice_array_2d():
 def test_slice_optimizations():
     #bar[:]
     expected = {('foo', 0): ('bar', 0)}
-    result, chunks = slice_array('foo', 'bar', [[100]], (slice(None,None,None),))
+    result, chunks = slice_array('foo', 'bar', [[100]], (slice(None, None, None),))
     assert expected == result
 
     #bar[:,:,:]
-    expected = {('foo', 0) : ('bar', 0),
-                ('foo', 1) : ('bar', 1),
-                ('foo', 2) : ('bar', 2)}
-    result, chunks = slice_array('foo', 'bar', [(100,1000,10000)],
-                        (slice(None,None,None),slice(None,None,None),
-                         slice(None,None,None)))
+    expected = {('foo', 0): ('bar', 0),
+                ('foo', 1): ('bar', 1),
+                ('foo', 2): ('bar', 2)}
+    result, chunks = slice_array('foo', 'bar', [(100, 1000, 10000)],
+                                 (slice(None, None, None),
+                                  slice(None, None, None),
+                                  slice(None, None, None)))
     assert expected == result
 
 
@@ -291,7 +279,6 @@ def test_take():
     assert dsk == expected
     assert chunks == ((4,), (20, 20))
 
-
     chunks, dsk = take('y', 'x', [(20, 20, 20, 20), (20, 20)], [5, 1, 37, 3], axis=1)
     expected = dict((('y', i, 0),
             (getitem,
@@ -386,14 +373,14 @@ def test_slice_stop_0():
     # from gh-125
     a = da.ones(10, chunks=(10,))[:0].compute()
     b = np.ones(10)[:0]
-    assert eq(a, b)
+    assert_eq(a, b)
 
 
 def test_slice_list_then_None():
     x = da.zeros(shape=(5, 5), chunks=(3, 3))
     y = x[[2, 1]][None]
 
-    assert eq(y, np.zeros((1, 2, 5)))
+    assert_eq(y, np.zeros((1, 2, 5)))
 
 
 class ReturnItem(object):
@@ -410,19 +397,19 @@ def test_slicing_exhaustively():
     # independent indexing along different axes
     indexers = [0, -2, I[:], I[:5], [0, 1], [0, 1, 2], [4, 2], I[::-1], None, I[:0], []]
     for i in indexers:
-        assert eq(x[i], a[i]), i
+        assert_eq(x[i], a[i]), i
         for j in indexers:
-            assert eq(x[i][:, j], a[i][:, j]), (i, j)
-            assert eq(x[:, i][j], a[:, i][j]), (i, j)
+            assert_eq(x[i][:, j], a[i][:, j]), (i, j)
+            assert_eq(x[:, i][j], a[:, i][j]), (i, j)
             for k in indexers:
-                assert eq(x[..., i][:, j][k], a[..., i][:, j][k]), (i, j, k)
+                assert_eq(x[..., i][:, j][k], a[..., i][:, j][k]), (i, j, k)
 
     # repeated indexing along the first axis
     first_indexers = [I[:], I[:5], np.arange(5), [3, 1, 4, 5, 0], np.arange(6) < 6]
     second_indexers = [0, -1, 3, I[:], I[:3], I[2:-1], [2, 4], [], I[:0]]
     for i in first_indexers:
         for j in second_indexers:
-            assert eq(x[i][j], a[i][j]), (i, j)
+            assert_eq(x[i][j], a[i][j]), (i, j)
 
 
 def test_slicing_with_negative_step_flops_keys():
@@ -431,7 +418,7 @@ def test_slicing_with_negative_step_flops_keys():
     assert (x.name, 1) in y.dask[(y.name, 0)]
     assert (x.name, 0) in y.dask[(y.name, 1)]
 
-    assert eq(y, np.arange(10)[:1:-1])
+    assert_eq(y, np.arange(10)[:1:-1])
 
     assert y.chunks == ((5, 3),)
 
@@ -445,13 +432,13 @@ def test_empty_slice():
     x = da.ones((5, 5), chunks=(2, 2), dtype='i4')
     y = x[:0]
 
-    assert eq(y, np.ones((5, 5), dtype='i4')[:0])
+    assert_eq(y, np.ones((5, 5), dtype='i4')[:0])
 
 
 def test_multiple_list_slicing():
     x = np.random.rand(6, 7, 8)
     a = da.from_array(x, chunks=(3, 3, 3))
-    assert eq(x[:, [0, 1, 2]][[0, 1]], a[:, [0, 1, 2]][[0, 1]])
+    assert_eq(x[:, [0, 1, 2]][[0, 1]], a[:, [0, 1, 2]][[0, 1]])
 
 
 def test_uneven_chunks():
@@ -507,3 +494,5 @@ def test_oob_check():
         x[6]
     with pytest.raises(IndexError):
         x[[6]]
+    with pytest.raises(IndexError):
+        x[0, 0]
