@@ -4,14 +4,14 @@ from collections import Iterable, Iterator, defaultdict
 from functools import wraps, partial
 import itertools
 import math
-from operator import getitem
 import os
 import types
 import uuid
+from random import Random
 from warnings import warn
 from distutils.version import LooseVersion
 
-from ..utils import ignoring
+from ..utils import ignoring, eq_strict
 
 from toolz import (merge, take, reduce, valmap, map, partition_all, filter,
                    remove, compose, curry, first, second, accumulate)
@@ -23,22 +23,25 @@ try:
     from cytoolz import (frequencies, merge_with, join, reduceby,
                          count, pluck, groupby, topk)
     if LooseVersion(cytoolz.__version__) > '0.7.3':
-        from cytoolz import accumulate
+        from cytoolz import accumulate  # noqa: F811
         _implement_accumulate = True
 except:
     from toolz import (frequencies, merge_with, join, reduceby,
                        count, pluck, groupby, topk)
 
 from ..base import Base, normalize_token, tokenize
-from ..compatibility import apply, unicode, urlopen
+from ..compatibility import apply, urlopen
 from ..context import _globals
 from ..core import list2, quote, istask, get_dependencies, reverse_dict
 from ..multiprocessing import get as mpget
 from ..optimize import fuse, cull, inline
-from ..utils import (infer_compression, open, system_encoding,
-                     takes_multiple_arguments, funcname, digit, insert)
+from ..utils import (open, system_encoding, takes_multiple_arguments, funcname,
+                     digit, insert, different_seeds)
+from ..bytes.core import write_bytes
+
 
 no_default = '__no__default__'
+no_result = '__no__result__'
 
 
 def lazify_task(task, start=True):
@@ -89,7 +92,10 @@ def inline_singleton_lists(dsk, dependencies=None):
 
     keys = [k for k, v in dsk.items()
             if istask(v) and v and v[0] is list and len(dependents[k]) == 1]
-    return inline(dsk, keys, inline_constants=False)
+    dsk = inline(dsk, keys, inline_constants=False)
+    for k in keys:
+        del dsk[k]
+    return dsk
 
 
 def optimize(dsk, keys, **kwargs):
@@ -101,7 +107,7 @@ def optimize(dsk, keys, **kwargs):
     return dsk5
 
 
-def to_textfiles(b, path, name_function=str, compression='infer',
+def to_textfiles(b, path, name_function=None, compression='infer',
                  encoding=system_encoding, compute=True):
     """ Write bag to disk, one filename per partition, one line per element
 
@@ -121,6 +127,8 @@ def to_textfiles(b, path, name_function=str, compression='infer',
 
     Use a globstring and a ``name_function=`` keyword argument.  The
     name_function function should expect an integer and produce a string.
+    Strings produced by name_function must preserve the order of their
+    respective partition indices.
 
     >>> from datetime import date, timedelta
     >>> def name(i):
@@ -146,49 +154,26 @@ def to_textfiles(b, path, name_function=str, compression='infer',
 
     **Compression**: Filenames with extensions corresponding to known
     compression algorithms (gz, bz2) will be compressed accordingly.
-    
-    **Bag Contents**: The bag calling ``to_textfiles`` _must_ be a bag of
-    text strings. For example, a bag of dictionaries could be written to 
-    JSON text files by mapping ``json.dumps``on to the bag first, and 
-    then calling ``to_textfiles``:
+
+    **Bag Contents**: The bag calling ``to_textfiles`` must be a bag of
+    text strings. For example, a bag of dictionaries could be written to
+    JSON text files by mapping ``json.dumps`` on to the bag first, and
+    then calling ``to_textfiles`` :
 
     >>> b_dict.map(json.dumps).to_textfiles("/path/to/data/*.json")  # doctest: +SKIP
-
     """
-    if isinstance(path, (str, unicode)):
-        if '*' in path:
-            paths = [path.replace('*', name_function(i))
-                     for i in range(b.npartitions)]
-        else:
-            paths = [os.path.join(path, '%s.part' % name_function(i))
-                     for i in range(b.npartitions)]
-    elif isinstance(path, (tuple, list, set)):
-        assert len(path) == b.npartitions
-        paths = path
-    else:
-        raise ValueError("""Path should be either"
-1.  A list of paths -- ['foo.json', 'bar.json', ...]
-2.  A directory -- 'foo/
-3.  A path with a * in it -- 'foo.*.json'""")
-
-    def get_compression(path, compression=compression):
-        if compression == 'infer':
-            compression = infer_compression(path)
-        return compression
-
-    name = 'to-textfiles-' + uuid.uuid4().hex
-    dsk = dict(((name, i), (write, (b.name, i), path, get_compression(path),
-                            encoding))
-               for i, path in enumerate(paths))
-
-    result = Bag(merge(b.dask, dsk), name, b.npartitions)
+    out = write_bytes(b.to_delayed(), path, name_function, compression,
+                      encoding=encoding)
     if compute:
-        result.compute()
+        from dask import compute
+        compute(*out)
     else:
-        return result
+        return out
 
 
 def finalize(results):
+    if not results:
+        return results
     if isinstance(results, Iterator):
         results = list(results)
     if isinstance(results[0], Iterable) and not isinstance(results[0], str):
@@ -216,7 +201,7 @@ def unpack_kwargs(kwargs):
         if isinstance(val, Item):
             dsk.update(val.dask)
             val = val.key
-        # TODO elif isinstance(val, Value):
+        # TODO elif isinstance(val, Delayed):
         elif isinstance(val, Base):
             raise NotImplementedError(
                 '%s not supported as kwarg value to Bag.map_partitions'
@@ -225,35 +210,82 @@ def unpack_kwargs(kwargs):
     return dsk, kw_pairs
 
 
+class StringAccessor(object):
+    """ String processing functions
+
+    Examples
+    --------
+
+    >>> import dask.bag as db
+    >>> b = db.from_sequence(['Alice Smith', 'Bob Jones', 'Charlie Smith'])
+    >>> list(b.str.lower())
+    ['alice smith', 'bob jones', 'charlie smith']
+
+    >>> list(b.str.match('*Smith'))
+    ['Alice Smith', 'Charlie Smith']
+
+    >>> list(b.str.split(' '))
+    [['Alice', 'Smith'], ['Bob', 'Jones'], ['Charlie', 'Smith']]
+    """
+    def __init__(self, bag):
+        self._bag = bag
+
+    def __dir__(self):
+        return sorted(set(dir(type(self)) + dir(str)))
+
+    def _strmap(self, key, *args, **kwargs):
+        return self._bag.map(lambda s: getattr(s, key)(*args, **kwargs))
+
+    def __getattr__(self, key):
+        try:
+            return object.__getattribute__(self, key)
+        except AttributeError:
+            if key in dir(str):
+                func = getattr(str, key)
+                return robust_wraps(func)(partial(self._strmap, key))
+            else:
+                raise
+
+    def match(self, pattern):
+        """ Filter strings by those that match a pattern
+
+        Examples
+        --------
+
+        >>> import dask.bag as db
+        >>> b = db.from_sequence(['Alice Smith', 'Bob Jones', 'Charlie Smith'])
+        >>> list(b.str.match('*Smith'))
+        ['Alice Smith', 'Charlie Smith']
+
+        See Also
+        --------
+        fnmatch.fnmatch
+        """
+        from fnmatch import fnmatch
+        return self._bag.filter(partial(fnmatch, pat=pattern))
+
+
+def robust_wraps(wrapper):
+    """ A weak version of wraps that only copies doc """
+    def _(wrapped):
+        wrapped.__doc__ = wrapper.__doc__
+        return wrapped
+    return _
+
+
 class Item(Base):
     _optimize = staticmethod(optimize)
     _default_get = staticmethod(mpget)
     _finalize = staticmethod(finalize_item)
 
-    @staticmethod
-    def from_imperative(value):
-        warn("Deprecation warning: moved to from_delayed")
-        return from_delayed(value)
-
     @staticmethod
     def from_delayed(value):
         """ Create bag item from a dask.delayed value
 
-        Parameters
-        ----------
-        value: a Value
-            A single dask.delayed.Value object, such as come from dask.do
-
-        Returns
-        -------
-        Item
-
-        Examples
-        --------
-        >>> b = db.Item.from_delayed(x)  # doctest: +SKIP
+        See ``dask.bag.from_delayed`` for details
         """
-        from dask.delayed import Value
-        assert isinstance(value, Value)
+        from dask.delayed import Delayed
+        assert isinstance(value, Delayed)
         return Item(value.dask, value.key)
 
     def __init__(self, dsk, key):
@@ -261,6 +293,16 @@ class Item(Base):
         self.key = key
         self.name = key
 
+    @property
+    def _args(self):
+        return (self.dask, self.key)
+
+    def __getstate__(self):
+        return self._args
+
+    def __setstate__(self, state):
+        self.dask, self.key = state
+
     def _keys(self):
         return [self.key]
 
@@ -271,17 +313,13 @@ class Item(Base):
 
     __int__ = __float__ = __complex__ = __bool__ = Base.compute
 
-    def to_imperative(self):
-        warn("Deprecation warning: moved to to_delayed")
-        return self.to_delayed()
-
     def to_delayed(self):
-        """ Convert bag item to dask Value
+        """ Convert bag item to dask Delayed
 
         Returns a single value.
         """
-        from dask.delayed import Value
-        return Value(self.key, [self.dask])
+        from dask.delayed import Delayed
+        return Delayed(self.key, [self.dask])
 
 
 class Bag(Base):
@@ -321,7 +359,6 @@ class Bag(Base):
         self.dask = dsk
         self.name = name
         self.npartitions = npartitions
-        self.str = StringAccessor(self)
 
     def __str__(self):
         name = self.name if len(self.name) < 10 else self.name[:7] + '...'
@@ -329,6 +366,8 @@ class Bag(Base):
 
     __repr__ = __str__
 
+    str = property(fget=StringAccessor)
+
     def map(self, func, **kwargs):
         """ Map a function across all elements in collection
 
@@ -381,6 +420,12 @@ class Bag(Base):
     def _args(self):
         return (self.dask, self.name, self.npartitions)
 
+    def __getstate__(self):
+        return self._args
+
+    def __setstate__(self, state):
+        self.dask, self.name, self.npartitions = state
+
     def filter(self, predicate):
         """ Filter elements in collection by a predicate function
 
@@ -398,6 +443,46 @@ class Bag(Base):
                    for i in range(self.npartitions))
         return type(self)(merge(self.dask, dsk), name, self.npartitions)
 
+    def _get_rs_predicate(self, seed, prob):
+        """ Return sampling filter predicate for the given seed.
+        """
+        random_state = Random(seed)
+        return lambda _: random_state.random() < prob
+
+    def random_sample(self, prob, random_state=None):
+        """ Return elements from bag with probability of ``prob``.
+
+        ``prob`` must be a number in the interval `[0, 1]`. All elements are
+        considered independently without replacement.
+
+        Providing an integer seed for ``random_state`` will result in
+        deterministic sampling. Given the same seed it will return the same
+        sample every time.
+
+        >>> import dask.bag as db
+        >>> b = db.from_sequence(range(5))
+        >>> list(b.random_sample(0.5, 42))
+        [1, 3]
+        >>> list(b.random_sample(0.5, 42))
+        [1, 3]
+        """
+        if not 0 <= prob <= 1:
+            raise ValueError('prob must be a number in the interval [0, 1]')
+        import numpy as np
+        if random_state is None:
+            random_state = np.random.randint(np.iinfo(np.int32).max)
+
+        name = 'random-sample-{0}'.format(tokenize(self, prob, random_state))
+        # we need to generate a different random seed for each partition or
+        # otherwise we would be selecting the exact same positions at each
+        # partition
+        seeds = different_seeds(self.npartitions, random_state)
+
+        dsk = dict(((name, i), (reify, (filter,
+                   self._get_rs_predicate(seed, prob), (self.name, i))))
+                   for i, seed in zip(range(self.npartitions), seeds))
+        return type(self)(merge(self.dask, dsk), name, self.npartitions)
+
     def remove(self, predicate):
         """ Remove elements in collection that match predicate
 
@@ -501,7 +586,7 @@ class Bag(Base):
         return tuple(self.pluck(i) for i in range(n))
 
     @wraps(to_textfiles)
-    def to_textfiles(self, path, name_function=str, compression='infer',
+    def to_textfiles(self, path, name_function=None, compression='infer',
                      encoding=system_encoding, compute=True):
         return to_textfiles(self, path, name_function, compression, encoding, compute)
 
@@ -635,7 +720,7 @@ class Bag(Base):
             split_every = self.npartitions
         token = tokenize(self, perpartition, aggregate, split_every)
         a = '%s-part-%s' % (name or funcname(perpartition), token)
-        dsk = dict(((a, i), (perpartition, (self.name, i)))
+        dsk = dict(((a, i), (empty_safe_apply, perpartition, (self.name, i)))
                    for i in range(self.npartitions))
         k = self.npartitions
         b = a
@@ -643,7 +728,9 @@ class Bag(Base):
         depth = 0
         while k > 1:
             c = fmt + str(depth)
-            dsk2 = dict(((c, i), (aggregate, [(b, j) for j in inds]))
+            dsk2 = dict(((c, i), (empty_safe_aggregate,
+                                   aggregate,
+                                   [(b, j) for j in inds]))
                         for i, inds in enumerate(partition_all(split_every,
                                                                range(k))))
             dsk.update(dsk2)
@@ -657,24 +744,24 @@ class Bag(Base):
         else:
             return Bag(merge(self.dask, dsk), b, 1)
 
-    @wraps(sum)
     def sum(self, split_every=None):
+        """ Sum all elements """
         return self.reduction(sum, sum, split_every=split_every)
 
-    @wraps(max)
     def max(self, split_every=None):
+        """ Maximum element """
         return self.reduction(max, max, split_every=split_every)
 
-    @wraps(min)
     def min(self, split_every=None):
+        """ Minimum element """
         return self.reduction(min, min, split_every=split_every)
 
-    @wraps(any)
     def any(self, split_every=None):
+        """ Are any of the elements truthy? """
         return self.reduction(any, any, split_every=split_every)
 
-    @wraps(all)
     def all(self, split_every=None):
+        """ Are all elements truthy? """
         return self.reduction(all, all, split_every=split_every)
 
     def count(self, split_every=None):
@@ -743,11 +830,11 @@ class Bag(Base):
         assert isinstance(other, Bag)
         name = 'product-' + tokenize(self, other)
         n, m = self.npartitions, other.npartitions
-        dsk = dict(((name, i*m + j),
+        dsk = dict(((name, i * m + j),
                    (list, (itertools.product, (self.name, i),
                                               (other.name, j))))
                    for i in range(n) for j in range(m))
-        return type(self)(merge(self.dask, other.dask, dsk), name, n*m)
+        return type(self)(merge(self.dask, other.dask, dsk), name, n * m)
 
     def foldby(self, key, binop, initial=no_default, combine=None,
                combine_initial=no_default):
@@ -850,19 +937,49 @@ class Bag(Base):
                                 list(dsk.keys())))}
         return type(self)(merge(self.dask, dsk, dsk2), b, 1)
 
-    def take(self, k, compute=True):
+    def take(self, k, npartitions=1, compute=True):
         """ Take the first k elements
 
-        Evaluates by default, use ``compute=False`` to avoid computation.
-        Only takes from the first partition
+        Parameters
+        ----------
+        k : int
+            The number of elements to return
+        npartitions : int, optional
+            Elements are only taken from the first ``npartitions``, with a
+            default of 1. If there are fewer than ``k`` rows in the first
+            ``npartitions`` a warning will be raised and any found rows
+            returned. Pass -1 to use all partitions.
+        compute : bool, optional
+            Whether to compute the result, default is True.
 
         >>> b = from_sequence(range(10))
         >>> b.take(3)  # doctest: +SKIP
         (0, 1, 2)
         """
-        name = 'take-' + tokenize(self, k)
-        dsk = {(name, 0): (list, (take, k, (self.name, 0)))}
+
+        if npartitions <= -1:
+            npartitions = self.npartitions
+        if npartitions > self.npartitions:
+            raise ValueError("only {} partitions, take "
+                "received {}".format(self.npartitions, npartitions))
+
+        token = tokenize(self, k, npartitions)
+        name = 'take-' + token
+
+        if npartitions > 1:
+            name_p = 'take-partial-' + token
+
+            dsk = {}
+            for i in range(npartitions):
+                dsk[(name_p, i)] = (list, (take, k, (self.name, i)))
+
+            concat = (toolz.concat, ([(name_p, i) for i in range(npartitions)]))
+            dsk[(name, 0)] = (safe_take, k, concat)
+        else:
+            dsk = {(name, 0): (safe_take, k, (self.name, 0))}
+
         b = Bag(merge(self.dask, dsk), name, 1)
+
         if compute:
             return tuple(b.compute())
         else:
@@ -943,16 +1060,25 @@ class Bag(Base):
     def to_dataframe(self, columns=None):
         """ Convert Bag to dask.dataframe
 
-        Bag should contain tuple or dict records.
-
-        Provide ``columns=`` keyword arg to specify column names.
+        Bag should contain tuples, dict records, or scalars.
 
         Index will not be particularly meaningful.  Use ``reindex`` afterwards
         if necessary.
 
+        Parameters
+        ----------
+        columns : pandas.DataFrame or list, optional
+            If a ``pandas.DataFrame``, it should mirror the column names and
+            dtypes of the output dataframe. If a list, it provides the desired
+            column names. If not provided or a list, a single element from
+            the first partition will be computed, triggering a potentially
+            expensive call to ``compute``. Providing a list is only useful for
+            selecting subset of columns, to avoid an internal compute call you
+            must provide a ``pandas.DataFrame`` as dask requires dtype knowledge
+            ahead of time.
+
         Examples
         --------
-
         >>> import dask.bag as db
         >>> b = db.from_sequence([{'name': 'Alice',   'balance': 100},
         ...                       {'name': 'Bob',     'balance': 200},
@@ -968,33 +1094,28 @@ class Bag(Base):
         """
         import pandas as pd
         import dask.dataframe as dd
-        if columns is None:
+        if isinstance(columns, pd.DataFrame):
+            meta = columns
+        else:
             head = self.take(1)[0]
-            if isinstance(head, dict):
-                columns = sorted(head)
-            elif isinstance(head, (tuple, list)):
-                columns = list(range(len(head)))
+            meta = pd.DataFrame([head], columns=columns)
+        columns = list(meta.columns)
         name = 'to_dataframe-' + tokenize(self, columns)
         DataFrame = partial(pd.DataFrame, columns=columns)
         dsk = dict(((name, i), (DataFrame, (list2, (self.name, i))))
                    for i in range(self.npartitions))
 
         divisions = [None] * (self.npartitions + 1)
-
         return dd.DataFrame(merge(optimize(self.dask, self._keys()), dsk),
-                            name, columns, divisions)
-
-    def to_imperative(self):
-        warn("Deprecation warning: moved to to_delayed")
-        return self.to_delayed()
+                            name, meta, divisions)
 
     def to_delayed(self):
-        """ Convert bag to dask Values
+        """ Convert bag to list of dask Delayed
 
-        Returns list of values, one value per partition.
+        Returns list of Delayed, one per partition.
         """
-        from dask.delayed import Value
-        return [Value(k, [self.dask]) for k in self._keys()]
+        from dask.delayed import Delayed
+        return [Delayed(k, [self.dask]) for k in self._keys()]
 
     def repartition(self, npartitions):
         """ Coalesce bag into fewer partitions
@@ -1007,18 +1128,25 @@ class Bag(Base):
             raise NotImplementedError(
               "Repartition only supports going to fewer partitions\n"
               " old: %d  new: %d" % (self.npartitions, npartitions))
-        size = self.npartitions / npartitions
-        L = [int(i * size) for i in range(npartitions + 1)]
-        name = 'repartition-%d-%s' % (npartitions, self.name)
-        dsk = dict(((name, i), (list,
-                                (toolz.concat,
-                                 [(self.name, j) for j in range(L[i], L[i + 1])]
-                                 )))
-                   for i in range(npartitions))
-        return Bag(merge(self.dask, dsk), name, npartitions)
+        npartitions_ratio = self.npartitions / npartitions
+        new_partitions_boundaries = [int(old_partition_index * npartitions_ratio)
+                                        for old_partition_index in range(npartitions + 1)]
+        new_name = 'repartition-%d-%s' % (npartitions, tokenize(self))
+        dsk = {(new_name, new_partition_index):
+                (list,
+                 (toolz.concat,
+                  [(self.name, old_partition_index)
+                    for old_partition_index in range(
+                        new_partitions_boundaries[new_partition_index],
+                        new_partitions_boundaries[new_partition_index + 1])]))
+                for new_partition_index in range(npartitions)}
+        return Bag(dsk=merge(self.dask, dsk), name=new_name, npartitions=npartitions)
 
     def accumulate(self, binop, initial=no_default):
-        """Repeatedly apply binary function to a sequence, accumulating results.
+        """ Repeatedly apply binary function to a sequence, accumulating results.
+
+        This assumes that the bag is ordered.  While this is typically the case
+        not all Dask.bag functions preserve this property.
 
         Examples
         --------
@@ -1073,7 +1201,7 @@ def partition(grouper, sequence, npartitions, p, nelements=2**20):
         d2 = defaultdict(list)
         for k, v in d.items():
             d2[abs(hash(k)) % npartitions].extend(v)
-        p.append(d2)
+        p.append(d2, fsync=True)
     return p
 
 
@@ -1083,15 +1211,6 @@ def collect(grouper, group, p, barrier_token):
     return list(d.items())
 
 
-def from_filenames(filenames, chunkbytes=None, compression='infer',
-                   encoding=system_encoding, linesep=os.linesep):
-    """ Deprecated.  See read_text """
-    warn("db.from_filenames is deprecated in favor of db.read_text")
-    from .text import read_text
-    return read_text(filenames, blocksize=chunkbytes, compression=compression,
-            encoding=encoding, linedelimiter=linesep)
-
-
 def write(data, filename, compression, encoding):
     dirname = os.path.dirname(filename)
     if not os.path.exists(dirname):
@@ -1162,7 +1281,7 @@ def from_sequence(seq, partition_size=None, npartitions=None):
 
     parts = list(partition_all(partition_size, seq))
     name = 'from_sequence-' + tokenize(seq, partition_size)
-    d = dict(((name, i), part) for i, part in enumerate(parts))
+    d = dict(((name, i), list(part)) for i, part in enumerate(parts))
     return Bag(d, name, len(d))
 
 
@@ -1216,6 +1335,8 @@ def load_castra_partition(castra, part, columns, index):
 def from_url(urls):
     """Create a dask.bag from a url
 
+    Examples
+    --------
     >>> a = from_url('http://raw.githubusercontent.com/dask/dask/master/README.rst')  # doctest: +SKIP
     >>> a.npartitions  # doctest: +SKIP
     1
@@ -1270,69 +1391,6 @@ def concat(bags):
     return Bag(merge(dsk, *[b.dask for b in bags]), name, len(dsk))
 
 
-class StringAccessor(object):
-    """ String processing functions
-
-    Examples
-    --------
-
-    >>> import dask.bag as db
-    >>> b = db.from_sequence(['Alice Smith', 'Bob Jones', 'Charlie Smith'])
-    >>> list(b.str.lower())
-    ['alice smith', 'bob jones', 'charlie smith']
-
-    >>> list(b.str.match('*Smith'))
-    ['Alice Smith', 'Charlie Smith']
-
-    >>> list(b.str.split(' '))
-    [['Alice', 'Smith'], ['Bob', 'Jones'], ['Charlie', 'Smith']]
-    """
-    def __init__(self, bag):
-        self._bag = bag
-
-    def __dir__(self):
-        return sorted(set(dir(type(self)) + dir(str)))
-
-    def _strmap(self, key, *args, **kwargs):
-        return self._bag.map(lambda s: getattr(s, key)(*args, **kwargs))
-
-    def __getattr__(self, key):
-        try:
-            return object.__getattribute__(self, key)
-        except AttributeError:
-            if key in dir(str):
-                func = getattr(str, key)
-                return robust_wraps(func)(partial(self._strmap, key))
-            else:
-                raise
-
-    def match(self, pattern):
-        """ Filter strings by those that match a pattern
-
-        Examples
-        --------
-
-        >>> import dask.bag as db
-        >>> b = db.from_sequence(['Alice Smith', 'Bob Jones', 'Charlie Smith'])
-        >>> list(b.str.match('*Smith'))
-        ['Alice Smith', 'Charlie Smith']
-
-        See Also
-        --------
-        fnmatch.fnmatch
-        """
-        from fnmatch import fnmatch
-        return self._bag.filter(partial(fnmatch, pat=pattern))
-
-
-def robust_wraps(wrapper):
-    """ A weak version of wraps that only copies doc """
-    def _(wrapped):
-        wrapped.__doc__ = wrapper.__doc__
-        return wrapped
-    return _
-
-
 def reify(seq):
     if isinstance(seq, Iterator):
         seq = list(seq)
@@ -1341,19 +1399,16 @@ def reify(seq):
     return seq
 
 
-def from_imperative(values):
-    warn("Deprecation warning: moved to from_delayed")
-    return from_delayed(values)
-
-
 def from_delayed(values):
     """ Create bag from many dask.delayed objects
 
+    These objects will become the partitions of the resulting Bag.  They should
+    evaluate to a ``list`` or some other concrete sequence.
+
     Parameters
     ----------
-    values: list of Values
-        An iterable of dask.delayed.Value objects, such as come from dask.do
-        These comprise the individual partitions of the resulting bag
+    values: list of delayed values
+        An iterable of dask Delayed objects.  Each evaluating to a list.
 
     Returns
     -------
@@ -1361,10 +1416,16 @@ def from_delayed(values):
 
     Examples
     --------
+    >>> x, y, z = [delayed(load_sequence_from_file)(fn)
+    ...             for fn in filenames] # doctest: +SKIP
     >>> b = from_delayed([x, y, z])  # doctest: +SKIP
+
+    See also
+    --------
+    dask.delayed
     """
-    from dask.delayed import Value
-    if isinstance(values, Value):
+    from dask.delayed import Delayed
+    if isinstance(values, Delayed):
         values = [values]
     dsk = merge(v.dask for v in values)
 
@@ -1443,7 +1504,8 @@ def bag_zip(*bags):
     >>> list(fizzbuzzz) # doctest: +SKIP
     [(0, 0), (3, 5), (6, 10), (9, 15), (12, 20), (15, 25), (18, 30)]
 
-    When what you really wanted was more along the lines of:
+    When what you really wanted was more along the lines of the following:
+
     >>> list(fizzbuzzz) # doctest: +SKIP
     [(0, 0), (3, None), (None, 5), (6, None), (None 10), (9, None),
     (12, None), (15, 15), (18, None), (None, 20), (None, 25), (None, 30)]
@@ -1489,7 +1551,6 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
 
     inputs = [tuple(digit(i, j, k) for j in range(stages))
               for i in range(k**stages)]
-    sinputs = set(inputs)
 
     b2 = b.map(lambda x: (hash(grouper(x)), x))
 
@@ -1513,7 +1574,7 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
 
         join = dict((('shuffle-join-' + token, stage, inp),
                      (list, (toolz.concat,
-                        [('shuffle-split-' + token, stage, inp[stage-1],
+                        [('shuffle-split-' + token, stage, inp[stage - 1],
                           insert(inp, stage - 1, j)) for j in range(k)])))
                      for inp in inputs)
         groups.append(group)
@@ -1563,3 +1624,25 @@ def groupby_disk(b, grouper, npartitions=None, blocksize=2**20):
 
     return type(b)(merge(b.dask, dsk1, dsk2, dsk3, dsk4), name,
                          npartitions)
+
+
+def empty_safe_apply(func, part):
+    part = list(part)
+    if part:
+        return func(part)
+    else:
+        return no_result
+
+
+def empty_safe_aggregate(func, parts):
+    parts2 = [p for p in parts if not eq_strict(p, no_result)]
+    return empty_safe_apply(func, parts2)
+
+
+def safe_take(n, b):
+    r = list(take(n, b))
+    if len(r) != n:
+        warn("Insufficient elements for `take`. {0} elements requested, "
+             "only {1} elements available. Try passing larger `npartitions` "
+             "to `take`.".format(n, len(r)))
+    return r
