@@ -30,6 +30,7 @@ L = list(range(5)) * 3
 
 b = Bag(dsk, 'x', 3)
 
+
 def iseven(x):
     return x % 2 == 0
 
@@ -65,10 +66,12 @@ class A(object):
     def __init__(self, a, b, c):
         pass
 
+
 class B(object):
     def __init__(self, a):
         pass
 
+
 def test_map_with_constructors():
     assert db.from_sequence([[1, 2, 3]]).map(A).compute()
     assert db.from_sequence([1, 2, 3]).map(B).compute()
@@ -88,9 +91,9 @@ def test_map_with_builtins():
     assert b.map(str).map(tuple).compute() == [('0',), ('1',), ('2',)]
     assert b.map(str).map(tuple).map(any).compute() == [True, True, True]
 
-    b2 = b.map(lambda n: [(n, n+1), (2*(n-1), -n)])
+    b2 = b.map(lambda n: [(n, n + 1), (2 * (n - 1), -n)])
     assert b2.map(dict).compute() == [{0: 1, -2: 0}, {1: 2, 0: -1}, {2: -2}]
-    assert b.map(lambda n: (n, n+1)).map(pow).compute() == [0, 1, 8]
+    assert b.map(lambda n: (n, n + 1)).map(pow).compute() == [0, 1, 8]
     assert b.map(bool).compute() == [False, True, True]
     assert db.from_sequence([(1, 'real'), ('1', 'real')]).map(hasattr).compute() == \
         [True, False]
@@ -172,6 +175,7 @@ def test_fold():
     assert c2.key == b.fold(add, initial=10).key
 
     c = db.from_sequence(range(5), npartitions=3)
+
     def binop(acc, x):
         acc = acc.copy()
         acc.add(x)
@@ -417,7 +421,7 @@ def test_inline_singleton_lists():
     out = {'c': (f, (list, 'a'), 1)}
     assert inline_singleton_lists(inp) == out
 
-    out = {'c': (f,        'a' , 1)}
+    out = {'c': (f, 'a', 1)}
     assert optimize(inp, ['c']) == out
 
     inp = {'b': (list, 'a'),
@@ -442,6 +446,7 @@ def test_take_npartitions():
     with pytest.raises(ValueError):
         b.take(1, npartitions=5)
 
+
 @pytest.mark.skipif(sys.version_info[:2] == (3,3),
     reason="Python3.3 uses pytest2.7.2, w/o warns method")
 def test_take_npartitions_warn():
@@ -721,6 +726,7 @@ def test_to_textfiles():
             assert 'xyz' in text
             f.close()
 
+
 def test_to_textfiles_name_function_preserves_order():
     seq = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p']
     b = db.from_sequence(seq, npartitions=16)
@@ -730,6 +736,7 @@ def test_to_textfiles_name_function_preserves_order():
         out = db.read_text(os.path.join(dn, "*"), encoding='ascii').map(str).map(str.strip).compute()
         assert seq == out
 
+
 @pytest.mark.skipif(sys.version_info[:2] == (3,3), reason="Python3.3 uses pytest2.7.2, w/o warns method")
 def test_to_textfiles_name_function_warn():
     seq = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p']
@@ -738,6 +745,7 @@ def test_to_textfiles_name_function_warn():
         with pytest.warns(None):
             a.to_textfiles(dn, name_function=str)
 
+
 def test_to_textfiles_encoding():
     b = db.from_sequence([u'汽车', u'苹果', u'天气'], npartitions=2)
     for ext, myopen in [('gz', GzipFile), ('bz2', BZ2File), ('', open)]:
@@ -864,29 +872,29 @@ def test_bag_compute_forward_kwargs():
 
 
 def test_to_delayed():
-    from dask.delayed import Value
+    from dask.delayed import Delayed
 
     b = db.from_sequence([1, 2, 3, 4, 5, 6], npartitions=3)
     a, b, c = b.map(inc).to_delayed()
-    assert all(isinstance(x, Value) for x in [a, b, c])
+    assert all(isinstance(x, Delayed) for x in [a, b, c])
     assert b.compute() == [4, 5]
 
     b = db.from_sequence([1, 2, 3, 4, 5, 6], npartitions=3)
     t = b.sum().to_delayed()
-    assert isinstance(t, Value)
+    assert isinstance(t, Delayed)
     assert t.compute() == 21
 
 
 def test_from_delayed():
-    from dask.delayed import value, do
-    a, b, c = value([1, 2, 3]), value([4, 5, 6]), value([7, 8, 9])
+    from dask.delayed import delayed
+    a, b, c = delayed([1, 2, 3]), delayed([4, 5, 6]), delayed([7, 8, 9])
     bb = from_delayed([a, b, c])
     assert bb.name == from_delayed([a, b, c]).name
 
     assert isinstance(bb, Bag)
     assert list(bb) == [1, 2, 3, 4, 5, 6, 7, 8, 9]
 
-    asum_value = do(lambda X: sum(X))(a)
+    asum_value = delayed(lambda X: sum(X))(a)
     asum_item = db.Item.from_delayed(asum_value)
     assert asum_value.compute() == asum_item.compute() == 6
 
@@ -922,6 +930,7 @@ def test_repartition():
     except NotImplementedError as e:
         assert '100' in str(e)
 
+
 @pytest.mark.skipif('not db.core._implement_accumulate')
 def test_accumulate():
     parts = [[1, 2, 3], [4, 5], [], [6, 7]]
@@ -1055,3 +1064,9 @@ def test_msgpack_unicode():
     b = db.from_sequence([{"a": 1}]).groupby("a")
     result = b.compute(get=dask.async.get_sync)
     assert dict(result) == {1: [{'a': 1}]}
+
+
+def test_bag_with_single_callable():
+    f = lambda: None
+    b = db.from_sequence([f])
+    assert list(b.compute(get=dask.get)) == [f]
