@@ -6,7 +6,6 @@ from operator import attrgetter
 import pickle
 import os
 import uuid
-import warnings
 
 from toolz import merge, groupby, curry, identity
 from toolz.functoolz import Compose
@@ -61,13 +60,6 @@ class Base(object):
         return visualize(self, filename=filename, format=format,
                          optimize_graph=optimize_graph, **kwargs)
 
-    def _visualize(self, filename='mydask', format=None, optimize_graph=False):
-        warn = DeprecationWarning("``_visualize`` is deprecated, use "
-                                  "``visualize`` instead.")
-        warnings.warn(warn)
-        return self.visualize(filename=filename, format=format,
-                              optimize_graph=optimize_graph)
-
     def compute(self, **kwargs):
         """Compute several dask collections at once.
 
@@ -249,13 +241,16 @@ def normalize_function(func):
         return (normalize_function(func.func), func.args, kws)
     else:
         try:
-            return pickle.dumps(func, protocol=0)
+            result = pickle.dumps(func, protocol=0)
+            if b'__main__' not in result:  # abort on dynamic functions
+                return result
         except:
-            try:
-                import cloudpickle
-                return cloudpickle.dumps(func, protocol=0)
-            except:
-                return str(func)
+            pass
+        try:
+            import cloudpickle
+            return cloudpickle.dumps(func, protocol=0)
+        except:
+            return str(func)
 
 
 normalize_token = Dispatch()
@@ -263,14 +258,17 @@ normalize_token.register((int, float, str, unicode, bytes, type(None), type,
                           slice),
                          identity)
 
+
 @partial(normalize_token.register, dict)
 def normalize_dict(d):
     return normalize_token(sorted(d.items(), key=str))
 
+
 @partial(normalize_token.register, (tuple, list, set))
 def normalize_seq(seq):
     return type(seq).__name__, list(map(normalize_token, seq))
 
+
 @partial(normalize_token.register, object)
 def normalize_object(o):
     if callable(o):
@@ -278,6 +276,7 @@ def normalize_object(o):
     else:
         return uuid.uuid4().hex
 
+
 @partial(normalize_token.register, Base)
 def normalize_base(b):
     return type(b).__name__, b.key
@@ -311,11 +310,12 @@ with ignoring(ImportError):
 
 with ignoring(ImportError):
     import numpy as np
+
     @partial(normalize_token.register, np.ndarray)
     def normalize_array(x):
         if not x.shape:
             return (str(x), x.dtype)
-        if hasattr(x, 'mode') and hasattr(x, 'filename'):
+        if hasattr(x, 'mode') and getattr(x, 'filename', None):
             return x.filename, os.path.getmtime(x.filename), x.dtype, x.shape
         if x.dtype.hasobject:
             try:
@@ -335,6 +335,7 @@ with ignoring(ImportError):
 
 with ignoring(ImportError):
     from collections import OrderedDict
+
     @partial(normalize_token.register, OrderedDict)
     def normalize_ordered_dict(d):
         return type(d).__name__, normalize_token(list(d.items()))
