@@ -15,13 +15,23 @@ from ..utils import system_encoding
 logger = logging.getLogger(__name__)
 
 
+def open_file_write(paths):
+    """ Open list of files using delayed """
+    out = [delayed(open)(path, 'wb') for path in paths]
+    return out
+
+
+def open_file_write_direct(path):
+    return open(path, 'wb')
+
+
 def read_bytes(path, delimiter=None, not_zero=False, blocksize=2**27,
         sample=True, compression=None):
     """ See dask.bytes.core.read_bytes for docstring """
     if '*' in path:
         filenames = list(map(os.path.abspath, sorted(glob(path))))
         sample, first = read_bytes(filenames[0], delimiter, not_zero,
-                                   blocksize, sample=True,
+                                   blocksize, sample=sample,
                                    compression=compression)
         rest = [read_bytes(f, delimiter, not_zero, blocksize, sample=False,
                            compression=compression)[1]
@@ -55,7 +65,7 @@ def read_bytes(path, delimiter=None, not_zero=False, blocksize=2**27,
                 nbytes = sample
             else:
                 nbytes = 10000
-            sample = read_block_from_file(path, 0, nbytes, None, compression)
+            sample = read_block_from_file(path, 0, nbytes, delimiter, compression)
 
         return sample, values
 
@@ -90,7 +100,7 @@ def open_files(path):
 from . import core
 core._read_bytes['file'] = read_bytes
 core._open_files['file'] = open_files
-
+core._open_files_write['file'] = open_file_write_direct
 
 if sys.version_info[0] >= 3:
     def open_text_files(path, encoding=system_encoding, errors='strict'):
