@@ -1,8 +1,10 @@
+from __future__ import print_function, division, absolute_import
+
 from .core import (DataFrame, Series, Index, _Frame, map_partitions,
                    repartition)
 from .io import (from_array, from_bcolz, from_array, from_bcolz,
                  from_pandas, from_dask_array, from_castra, read_hdf,
-                 from_imperative, from_delayed)
+                 from_delayed)
 from .optimize import optimize
 from .multi import merge, concat, melt
 from .rolling import (rolling_count, rolling_sum, rolling_mean, rolling_median,
@@ -11,4 +13,5 @@ from .rolling import (rolling_count, rolling_sum, rolling_mean, rolling_median,
                       rolling_window)
 from ..base import compute
 from .csv import read_csv
+from .reshape import get_dummies
 from . import demo
