@@ -12,14 +12,15 @@ __all__ = ['make_timeseries']
 def make_float(n, rstate):
     return rstate.rand(n) * 2 - 1
 
+
 def make_int(n, rstate):
     return rstate.poisson(1000, size=n)
 
 
 names = ['Alice', 'Bob', 'Charlie', 'Dan', 'Edith', 'Frank', 'George',
-'Hannah', 'Ingrid', 'Jerry', 'Kevin', 'Laura', 'Michael', 'Norbert', 'Oliver',
-'Patricia', 'Quinn', 'Ray', 'Sarah', 'Tim', 'Ursula', 'Victor', 'Wendy',
-'Xavier', 'Yvonne', 'Zelda']
+         'Hannah', 'Ingrid', 'Jerry', 'Kevin', 'Laura', 'Michael', 'Norbert',
+         'Oliver', 'Patricia', 'Quinn', 'Ray', 'Sarah', 'Tim', 'Ursula',
+         'Victor', 'Wendy', 'Xavier', 'Yvonne', 'Zelda']
 
 
 def make_string(n, rstate):
@@ -86,5 +87,5 @@ def make_timeseries(start, end, dtypes, freq, partition_freq, seed=None):
     dsk = dict(((name, i), (make_timeseries_part, divisions[i], divisions[i + 1],
                                                  dtypes, freq, seeds[i]))
                 for i in range(len(divisions) - 1))
-    head = make_timeseries_part('2000','2000', dtypes, '1H', 1)
+    head = make_timeseries_part('2000', '2000', dtypes, '1H', 1)
     return DataFrame(dsk, name, head, divisions)
