@@ -56,6 +56,8 @@ We proceed with hash joins in the following stages:
 from __future__ import absolute_import, division, print_function
 
 from bisect import bisect_left, bisect_right
+from functools import wraps
+from warnings import warn
 
 from toolz import merge_sorted, unique, partial, first
 import toolz
@@ -64,8 +66,8 @@ import pandas as pd
 
 from ..base import tokenize
 from ..compatibility import apply
-from .core import (_Frame, Scalar, DataFrame, map_partitions,
-                   Index, _maybe_from_pandas)
+from .core import (_Frame, DataFrame, map_partitions,
+                   Index, _maybe_from_pandas, new_dd_object)
 from .io import from_pandas
 from .shuffle import shuffle, rearrange_by_divisions
 
@@ -90,7 +92,6 @@ def align_partitions(*dfs):
     dfs: sequence of dd.DataFrame, dd.Series and dd.base.Scalar
         Sequence of dataframes to be aligned on their index
 
-
     Returns
     -------
     dfs: sequence of dd.DataFrame, dd.Series and dd.base.Scalar
@@ -104,6 +105,10 @@ def align_partitions(*dfs):
     dfs1 = [df for df in dfs if isinstance(df, _Frame)]
     if len(dfs) == 0:
         raise ValueError("dfs contains no DataFrame and Series")
+    if not all(df.known_divisions for df in dfs1):
+        raise ValueError("Not all divisions are known, can't align "
+                         "partitions. Please use `set_index` or "
+                         "`set_partition` to set the index.")
     divisions = list(unique(merge_sorted(*[df.divisions for df in dfs1])))
     dfs2 = [df.repartition(divisions, force=True)
             if isinstance(df, _Frame) else df for df in dfs]
@@ -121,29 +126,26 @@ def align_partitions(*dfs):
                     inds[i] += 1
                 else:
                     L.append(None)
-            else: # Scalar has no divisions
+            else:    # Scalar has no divisions
                 L.append(None)
         result.append(L)
     return dfs2, tuple(divisions), result
 
 
 def _maybe_align_partitions(args):
-    """ Align DataFrame blocks if divisions are different """
-    # passed to align_partitions
-    indexer, dasks = zip(*[x for x in enumerate(args)
-                           if isinstance(x[1], (_Frame, Scalar))])
+    """Align DataFrame blocks if divisions are different.
 
-    # to get current divisions
-    dfs = [df for df in dasks if isinstance(df, _Frame)]
-    if len(dfs) == 0:
-        # no need to align
+    Note that if all divisions are unknown, but have equal npartitions, then
+    they will be passed through unchanged. This is different than
+    `align_partitions`, which will fail if divisions aren't all known"""
+    dfs = [df for df in args if isinstance(df, _Frame)]
+    if not dfs:
         return args
 
     divisions = dfs[0].divisions
     if not all(df.divisions == divisions for df in dfs):
-        dasks, _, _ = align_partitions(*dasks)
-        for i, d in zip(indexer, dasks):
-            args[i] = d
+        dfs2 = iter(align_partitions(*dfs)[0])
+        return [a if not isinstance(a, _Frame) else next(dfs2) for a in args]
     return args
 
 
@@ -198,15 +200,19 @@ def require(divisions, parts, required=None):
 
 required = {'left': [0], 'right': [1], 'inner': [0, 1], 'outer': []}
 
-def join_indexed_dataframes(lhs, rhs, how='left', lsuffix='', rsuffix=''):
-    """ Join two partitiond dataframes along their index """
+
+def merge_indexed_dataframes(lhs, rhs, how='left', lsuffix='', rsuffix='',
+                             indicator=False):
+    """ Join two partitioned dataframes along their index """
+
     (lhs, rhs), divisions, parts = align_partitions(lhs, rhs)
     divisions, parts = require(divisions, parts, required[how])
 
-    left_empty = lhs._pd_nonempty
-    right_empty = rhs._pd_nonempty
+    left_empty = lhs._meta
+    right_empty = rhs._meta
 
-    name = 'join-indexed-' + tokenize(lhs, rhs, how, lsuffix, rsuffix)
+    name = 'join-indexed-' + tokenize(lhs, rhs, how, lsuffix, rsuffix,
+                                      indicator)
 
     dsk = dict()
     for i, (a, b) in enumerate(parts):
@@ -215,38 +221,38 @@ def join_indexed_dataframes(lhs, rhs, how='left', lsuffix='', rsuffix=''):
         if b is None and how in ('left', 'outer'):
             b = right_empty
 
-        dsk[(name, i)] = (pd.DataFrame.join, a, b, None, how,
-                          lsuffix, rsuffix)
+        dsk[(name, i)] = (_pdmerge, a, b, how, None, None, True, True,
+                          indicator, (lsuffix, rsuffix), left_empty,
+                          right_empty)
 
-    # dummy result
-    dummy = left_empty.join(right_empty, on=None, how=how,
-                            lsuffix=lsuffix, rsuffix=rsuffix)
+    meta = pd.merge(lhs._meta_nonempty, rhs._meta_nonempty, how=how,
+                    left_index=True, right_index=True,
+                    suffixes=(lsuffix, rsuffix), indicator=indicator)
     return DataFrame(toolz.merge(lhs.dask, rhs.dask, dsk),
-                     name, dummy, divisions)
+                     name, meta, divisions)
 
 
 def _pdmerge(left, right, how, left_on, right_on,
-             left_index, right_index, suffixes,
-             default_left_columns, default_right_columns):
+             left_index, right_index, indicator, suffixes,
+             default_left, default_right):
 
     if not len(left):
-        left = pd.DataFrame(columns=default_left_columns)
+        left = default_left
 
     if not len(right):
-        right = pd.DataFrame(columns=default_right_columns)
+        right = default_right
 
-    result = pd.merge(left, right, how=how,
-                      left_on=left_on, right_on=right_on,
-                      left_index=left_index, right_index=right_index,
-                      suffixes=suffixes)
-    return result
+    return pd.merge(left, right, how=how, left_on=left_on, right_on=right_on,
+                    left_index=left_index, right_index=right_index,
+                    suffixes=suffixes, indicator=indicator)
 
 
 shuffle_func = shuffle  # name sometimes conflicts with keyword argument
 
 
 def hash_join(lhs, left_on, rhs, right_on, how='inner',
-              npartitions=None, suffixes=('_x', '_y'), shuffle=None):
+              npartitions=None, suffixes=('_x', '_y'), shuffle=None,
+              indicator=False):
     """ Join two DataFrames on particular columns with hash join
 
     This shuffles both datasets on the joined column and then performs an
@@ -273,14 +279,14 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
         right_index = False
 
     # dummy result
-    dummy = pd.merge(lhs._pd_nonempty, rhs._pd_nonempty, how, None,
-                     left_on=left_on, right_on=right_on,
-                     left_index=left_index, right_index=right_index,
-                     suffixes=suffixes)
+    meta = pd.merge(lhs._meta_nonempty, rhs._meta_nonempty, how=how,
+                    left_on=left_on, right_on=right_on,
+                    left_index=left_index, right_index=right_index,
+                    suffixes=suffixes, indicator=indicator)
 
     merger = partial(_pdmerge, suffixes=suffixes,
-                     default_left_columns=list(lhs.columns),
-                     default_right_columns=list(rhs.columns))
+                     default_left=lhs._meta,
+                     default_right=rhs._meta)
 
     if isinstance(left_on, list):
         left_on = (list, tuple(left_on))
@@ -288,21 +294,21 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
         right_on = (list, tuple(right_on))
 
     token = tokenize(lhs2, left_on, rhs2, right_on, left_index, right_index,
-                     how, npartitions, suffixes, shuffle)
+                     how, npartitions, suffixes, shuffle, indicator)
     name = 'hash-join-' + token
 
     dsk = dict(((name, i), (merger, (lhs2._name, i), (rhs2._name, i),
                             how, left_on, right_on,
-                            left_index, right_index))
+                            left_index, right_index, indicator))
                 for i in range(npartitions))
 
     divisions = [None] * (npartitions + 1)
     return DataFrame(toolz.merge(lhs2.dask, rhs2.dask, dsk),
-                     name, dummy, divisions)
+                     name, meta, divisions)
 
 
 def single_partition_join(left, right, **kwargs):
-    meta = pd.merge(left._pd_nonempty, right._pd_nonempty, **kwargs)
+    meta = pd.merge(left._meta_nonempty, right._meta_nonempty, **kwargs)
     name = 'merge-' + tokenize(left, right, **kwargs)
     if left.npartitions == 1:
         left_key = first(left._keys())
@@ -355,6 +361,24 @@ def _pdconcat(dfs, axis=0, join='outer'):
     return pd.concat(dfs, axis=axis, join=join)
 
 
+def concat_and_check(dfs):
+    if len(set(map(len, dfs))) != 1:
+        raise ValueError("Concatenated DataFrames of different lengths")
+    return pd.concat(dfs, axis=1)
+
+
+def concat_unindexed_dataframes(dfs):
+    name = 'concat-' + tokenize(*dfs)
+
+    dsk = {(name, i): (concat_and_check, [(df._name, i) for df in dfs])
+            for i in range(dfs[0].npartitions)}
+
+    meta = pd.concat([df._meta for df in dfs], axis=1)
+
+    return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
+                         name, meta, dfs[0].divisions)
+
+
 def concat_indexed_dataframes(dfs, axis=0, join='outer'):
     """ Concatenate indexed dataframes together along the index """
 
@@ -362,11 +386,11 @@ def concat_indexed_dataframes(dfs, axis=0, join='outer'):
         raise ValueError("'join' must be 'inner' or 'outer'")
 
     from dask.dataframe.core import _emulate
-    dummy = _emulate(pd.concat, dfs, axis=axis, join=join)
+    meta = _emulate(pd.concat, dfs, axis=axis, join=join)
 
     dfs = _maybe_from_pandas(dfs)
     dfs2, divisions, parts = align_partitions(*dfs)
-    empties = [df._pd for df in dfs]
+    empties = [df._meta for df in dfs]
 
     parts2 = [[df if df is not None else empty
                for df, empty in zip(part, empties)]
@@ -376,13 +400,14 @@ def concat_indexed_dataframes(dfs, axis=0, join='outer'):
     dsk = dict(((name, i), (_pdconcat, part, axis, join))
                 for i, part in enumerate(parts2))
 
-    return _Frame(toolz.merge(dsk, *[df.dask for df in dfs2]),
-                  name, dummy, divisions)
+    return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs2]),
+                         name, meta, divisions)
 
 
+@wraps(pd.merge)
 def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
           left_index=False, right_index=False, suffixes=('_x', '_y'),
-          npartitions=None, shuffle=None, max_branch=None):
+          indicator=False, npartitions=None, shuffle=None, max_branch=None):
     for o in [on, left_on, right_on]:
         if isinstance(o, _Frame):
             raise NotImplementedError(
@@ -397,10 +422,11 @@ def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
         on = None
 
     if (isinstance(left, (pd.Series, pd.DataFrame)) and
-        isinstance(right, (pd.Series, pd.DataFrame))):
+            isinstance(right, (pd.Series, pd.DataFrame))):
         return pd.merge(left, right, how=how, on=on, left_on=left_on,
                         right_on=right_on, left_index=left_index,
-                        right_index=right_index, suffixes=suffixes)
+                        right_index=right_index, suffixes=suffixes,
+                        indicator=indicator)
 
     # Transform pandas objects into dask.dataframe objects
     if isinstance(left, (pd.Series, pd.DataFrame)):
@@ -421,41 +447,47 @@ def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
 
     # Both sides indexed
     if (left_index and left.known_divisions and
-        right_index and right.known_divisions):  # Do indexed join
-        return join_indexed_dataframes(left, right, how=how,
-                                       lsuffix=suffixes[0], rsuffix=suffixes[1])
+            right_index and right.known_divisions):  # Do indexed join
+        return merge_indexed_dataframes(left, right, how=how,
+                                        lsuffix=suffixes[0],
+                                        rsuffix=suffixes[1],
+                                        indicator=indicator)
 
     # Single partition on one side
     elif (left.npartitions == 1 and how in ('inner', 'right') or
           right.npartitions == 1 and how in ('inner', 'left')):
         return single_partition_join(left, right, how=how, right_on=right_on,
-                left_on=left_on, left_index=left_index,
-                right_index=right_index, suffixes=suffixes)
+                                     left_on=left_on, left_index=left_index,
+                                     right_index=right_index,
+                                     suffixes=suffixes, indicator=indicator)
 
     # One side is indexed, the other not
     elif (left_index and left.known_divisions and not right_index or
           right_index and right.known_divisions and not left_index):
-        left_empty = left._pd_nonempty
-        right_empty = right._pd_nonempty
-        dummy = pd.merge(left_empty, right_empty, how=how, on=on, left_on=left_on,
-                right_on=right_on, left_index=left_index, right_index=right_index,
-                suffixes=suffixes)
+        left_empty = left._meta_nonempty
+        right_empty = right._meta_nonempty
+        meta = pd.merge(left_empty, right_empty, how=how, on=on,
+                        left_on=left_on, right_on=right_on,
+                        left_index=left_index, right_index=right_index,
+                        suffixes=suffixes, indicator=indicator)
         if left_index and left.known_divisions:
             right = rearrange_by_divisions(right, right_on, left.divisions,
-                    max_branch, shuffle=shuffle)
+                                           max_branch, shuffle=shuffle)
             left = left.clear_divisions()
         elif right_index and right.known_divisions:
             left = rearrange_by_divisions(left, left_on, right.divisions,
-                    max_branch, shuffle=shuffle)
+                                          max_branch, shuffle=shuffle)
             right = right.clear_divisions()
-        return map_partitions(pd.merge, dummy, left, right, how=how, on=on,
-                left_on=left_on, right_on=right_on, left_index=left_index,
-                right_index=right_index, suffixes=suffixes)
+        return map_partitions(pd.merge, left, right, meta=meta, how=how, on=on,
+                              left_on=left_on, right_on=right_on,
+                              left_index=left_index, right_index=right_index,
+                              suffixes=suffixes, indicator=indicator)
     # Catch all hash join
     else:
         return hash_join(left, left.index if left_index else left_on,
                          right, right.index if right_index else right_on,
-                         how, npartitions, suffixes, shuffle=shuffle)
+                         how, npartitions, suffixes, shuffle=shuffle,
+                         indicator=indicator)
 
 
 ###############################################################
@@ -467,14 +499,14 @@ def _concat_dfs(dfs, name, join='outer'):
     dsk = dict()
     i = 0
 
-    empties = [df._pd for df in dfs]
-    dummy = pd.concat(empties, axis=0, join=join)
+    empties = [df._meta for df in dfs]
+    meta = pd.concat(empties, axis=0, join=join)
 
-    if isinstance(dummy, pd.Series):
+    if isinstance(meta, pd.Series):
         # in this case, input must be all Series. No need to care DataFrame.
         columns = pd.Index([])
     else:
-        columns = dummy.columns
+        columns = meta.columns
         if len(columns) == 0:
             raise ValueError('Failed to concat, no columns remain')
 
@@ -490,7 +522,7 @@ def _concat_dfs(dfs, name, join='outer'):
             dsk[(name, i)] = key
             i += 1
 
-    return dsk, dummy
+    return dsk, meta
 
 
 def concat(dfs, axis=0, join='outer', interleave_partitions=False):
@@ -574,7 +606,6 @@ def concat(dfs, axis=0, join='outer', interleave_partitions=False):
     dasks = [df for df in dfs if isinstance(df, _Frame)]
 
     if all(df.known_divisions for df in dasks):
-
         dfs = _maybe_from_pandas(dfs)
         if axis == 1:
             return concat_indexed_dataframes(dfs, axis=axis, join=join)
@@ -586,15 +617,15 @@ def concat(dfs, axis=0, join='outer', interleave_partitions=False):
             if all(dfs[i].divisions[-1] < dfs[i + 1].divisions[0]
                    for i in range(len(dfs) - 1)):
                 name = 'concat-{0}'.format(tokenize(*dfs))
-                dsk, dummy = _concat_dfs(dfs, name, join=join)
+                dsk, meta = _concat_dfs(dfs, name, join=join)
 
                 divisions = []
                 for df in dfs[:-1]:
                     # remove last to concatenate with next
                     divisions += df.divisions[:-1]
                 divisions += dfs[-1].divisions
-                return _Frame(toolz.merge(dsk, *[df.dask for df in dfs]),
-                              name, dummy, divisions)
+                return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
+                                     name, meta, divisions)
             else:
                 if interleave_partitions:
                     return concat_indexed_dataframes(dfs, join=join)
@@ -602,27 +633,35 @@ def concat(dfs, axis=0, join='outer', interleave_partitions=False):
                 raise ValueError('All inputs have known divisions which cannot '
                                  'be concatenated in order. Specify '
                                  'interleave_partitions=True to ignore order')
+    elif (axis == 1 and
+          len(dasks) == len(dfs) and
+          all(not df.known_divisions for df in dfs) and
+          len({df.npartitions for df in dasks}) == 1):
+        warn("Concatenating dataframes with unknown divisions.\n"
+             "We're assuming that the indexes of each dataframes are aligned\n."
+             "This assumption is not generally safe.")
+        return concat_unindexed_dataframes(dfs)
 
     else:
         if axis == 1:
-             raise ValueError('Unable to concatenate DataFrame with unknown '
-                              'division specifying axis=1')
+            raise ValueError('Unable to concatenate DataFrame with unknown '
+                             'division specifying axis=1')
         else:
             # concat will not regard Series as row
             dfs = _maybe_from_pandas(dfs)
             name = 'concat-{0}'.format(tokenize(*dfs))
-            dsk, dummy = _concat_dfs(dfs, name, join=join)
+            dsk, meta = _concat_dfs(dfs, name, join=join)
 
             divisions = [None] * (sum([df.npartitions for df in dfs]) + 1)
-            return _Frame(toolz.merge(dsk, *[df.dask for df in dfs]),
-                          name, dummy, divisions)
-
+            return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
+                                 name, meta, divisions)
 
 
 ###############################################################
 # Append
 ###############################################################
 
+
 def _append(df, other, divisions):
     """ Internal function to append 2 dd.DataFrame/Series instances """
     # ToDo: might be possible to merge the logic to concat,
@@ -636,8 +675,8 @@ def _append(df, other, divisions):
     for j in range(other.npartitions):
         dsk[(name, npart + j)] = (other._name, j)
     dsk = toolz.merge(dsk, df.dask, other.dask)
-    dummy = df._pd.append(other._pd)
-    return _Frame(dsk, name, dummy, divisions)
+    meta = df._meta.append(other._meta)
+    return new_dd_object(dsk, name, meta, divisions)
 
 
 ###############################################################
@@ -649,7 +688,7 @@ def melt(frame, id_vars=None, value_vars=None, var_name=None,
 
     from dask.dataframe.core import no_default
 
-    return frame.map_partitions(pd.melt, no_default, id_vars=id_vars,
+    return frame.map_partitions(pd.melt, meta=no_default, id_vars=id_vars,
                                 value_vars=value_vars,
                                 var_name=var_name, value_name=value_name,
                                 col_level=col_level, token='melt')
