@@ -1,8 +1,9 @@
 import dask.dataframe as dd
 import numpy as np
 import pandas as pd
-from dask.dataframe.multi import (align_partitions, join_indexed_dataframes,
-        hash_join, concat_indexed_dataframes)
+from dask.dataframe.multi import (align_partitions, merge_indexed_dataframes,
+                                  hash_join, concat_indexed_dataframes,
+                                  _maybe_align_partitions)
 import pandas.util.testing as tm
 from dask.async import get_sync
 from dask.dataframe.utils import eq
@@ -19,7 +20,7 @@ def test_align_partitions():
                      index=[30, 70, 80, 100])
     b = dd.repartition(B, [30, 80, 100])
 
-    s = dd.core.Scalar({('s', 0): 10}, 's')
+    s = dd.core.Scalar({('s', 0): 10}, 's', 'i8')
 
     (aa, bb), divisions, L = align_partitions(a, b)
 
@@ -59,7 +60,7 @@ def test_align_partitions():
                      (dd.from_pandas(ldf, 2), dd.from_pandas(rdf, 2)),
                      (dd.from_pandas(ldf, 2), dd.from_pandas(rdf, 3)),
                      (dd.from_pandas(ldf, 3), dd.from_pandas(rdf, 2))]:
-        (lresult, rresult), div, parts = dd.multi.align_partitions(lhs, rhs)
+        (lresult, rresult), div, parts = align_partitions(lhs, rhs)
         assert eq(lresult, ldf)
         assert eq(rresult, rdf)
 
@@ -75,12 +76,79 @@ def test_align_partitions():
                      (dd.from_pandas(ldf, 2), dd.from_pandas(rdf, 2)),
                      (dd.from_pandas(ldf, 2), dd.from_pandas(rdf, 3)),
                      (dd.from_pandas(ldf, 3), dd.from_pandas(rdf, 2))]:
-        (lresult, rresult), div, parts = dd.multi.align_partitions(lhs, rhs)
+        (lresult, rresult), div, parts = align_partitions(lhs, rhs)
         assert eq(lresult, ldf)
         assert eq(rresult, rdf)
 
 
-def test_join_indexed_dataframe_to_indexed_dataframe():
+def test_align_partitions_unknown_divisions():
+    df = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
+                       'b': [7, 6, 5, 4, 3, 2, 1]})
+    # One known, one unknown
+    ddf = dd.from_pandas(df, npartitions=2)
+    ddf2 = dd.from_pandas(df, npartitions=2, sort=False)
+    assert not ddf2.known_divisions
+
+    with pytest.raises(ValueError):
+        align_partitions(ddf, ddf2)
+
+    # Both unknown
+    ddf = dd.from_pandas(df + 1, npartitions=2, sort=False)
+    ddf2 = dd.from_pandas(df, npartitions=2, sort=False)
+    assert not ddf.known_divisions
+    assert not ddf2.known_divisions
+
+    with pytest.raises(ValueError):
+        align_partitions(ddf, ddf2)
+
+
+def test__maybe_align_partitions():
+    df = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
+                       'b': [7, 6, 5, 4, 3, 2, 1]})
+    # Both known, same divisions
+    ddf = dd.from_pandas(df + 1, npartitions=2)
+    ddf2 = dd.from_pandas(df, npartitions=2)
+
+    a, b = _maybe_align_partitions([ddf, ddf2])
+    assert a is ddf
+    assert b is ddf2
+
+    # Both unknown, same divisions
+    ddf = dd.from_pandas(df + 1, npartitions=2, sort=False)
+    ddf2 = dd.from_pandas(df, npartitions=2, sort=False)
+    assert not ddf.known_divisions
+    assert not ddf2.known_divisions
+
+    a, b = _maybe_align_partitions([ddf, ddf2])
+    assert a is ddf
+    assert b is ddf2
+
+    # Both known, different divisions
+    ddf = dd.from_pandas(df + 1, npartitions=2)
+    ddf2 = dd.from_pandas(df, npartitions=3)
+
+    a, b = _maybe_align_partitions([ddf, ddf2])
+    assert a.divisions == b.divisions
+
+    # Both unknown, different divisions
+    ddf = dd.from_pandas(df + 1, npartitions=2, sort=False)
+    ddf2 = dd.from_pandas(df, npartitions=3, sort=False)
+    assert not ddf.known_divisions
+    assert not ddf2.known_divisions
+
+    with pytest.raises(ValueError):
+        _maybe_align_partitions([ddf, ddf2])
+
+    # One known, one unknown
+    ddf = dd.from_pandas(df, npartitions=2)
+    ddf2 = dd.from_pandas(df, npartitions=2, sort=False)
+    assert not ddf2.known_divisions
+
+    with pytest.raises(ValueError):
+        _maybe_align_partitions([ddf, ddf2])
+
+
+def test_merge_indexed_dataframe_to_indexed_dataframe():
     A = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6]},
                      index=[1, 2, 3, 4, 6, 7])
     a = dd.repartition(A, [1, 4, 7])
@@ -89,55 +157,55 @@ def test_join_indexed_dataframe_to_indexed_dataframe():
                      index=[1, 2, 4, 5, 6, 8])
     b = dd.repartition(B, [1, 2, 5, 8])
 
-    c = join_indexed_dataframes(a, b, how='left')
+    c = merge_indexed_dataframes(a, b, how='left')
     assert c.divisions[0] == a.divisions[0]
     assert c.divisions[-1] == max(a.divisions + b.divisions)
     assert eq(c, A.join(B))
 
-    c = join_indexed_dataframes(a, b, how='right')
+    c = merge_indexed_dataframes(a, b, how='right')
     assert c.divisions[0] == b.divisions[0]
     assert c.divisions[-1] == b.divisions[-1]
     assert eq(c, A.join(B, how='right'))
 
-    c = join_indexed_dataframes(a, b, how='inner')
+    c = merge_indexed_dataframes(a, b, how='inner')
     assert c.divisions[0] == 1
     assert c.divisions[-1] == max(a.divisions + b.divisions)
     assert eq(c.compute(), A.join(B, how='inner'))
 
-    c = join_indexed_dataframes(a, b, how='outer')
+    c = merge_indexed_dataframes(a, b, how='outer')
     assert c.divisions[0] == 1
     assert c.divisions[-1] == 8
     assert eq(c.compute(), A.join(B, how='outer'))
 
-    assert sorted(join_indexed_dataframes(a, b, how='inner').dask) == \
-           sorted(join_indexed_dataframes(a, b, how='inner').dask)
-    assert sorted(join_indexed_dataframes(a, b, how='inner').dask) != \
-           sorted(join_indexed_dataframes(a, b, how='outer').dask)
+    assert sorted(merge_indexed_dataframes(a, b, how='inner').dask) == \
+           sorted(merge_indexed_dataframes(a, b, how='inner').dask)
+    assert sorted(merge_indexed_dataframes(a, b, how='inner').dask) != \
+           sorted(merge_indexed_dataframes(a, b, how='outer').dask)
 
 
-def list_eq(a, b):
-    if isinstance(a, dd.DataFrame):
-        a = a.compute(get=get_sync)
-    if isinstance(b, dd.DataFrame):
-        b = b.compute(get=get_sync)
+def list_eq(aa, bb):
+    if isinstance(aa, dd.DataFrame):
+        a = aa.compute(get=get_sync)
+    else:
+        a = aa
+    if isinstance(bb, dd.DataFrame):
+        b = bb.compute(get=get_sync)
+    else:
+        b = bb
     tm.assert_index_equal(a.columns, b.columns)
 
-    # ToDo: As of pandas 0,17, tm.assert_numpy_array_equal can
-    # compare arrays include NaN. This logic can be replaced
-    for c in a.columns:
-        if a[c].dtype in (int, float):
-            a[c] = a[c].fillna(100)
-            b[c] = b[c].fillna(100)
-        else:
-            a[c] = a[c].fillna('NAN')
-            b[c] = b[c].fillna('NAN')
-    av = sorted(a.values.tolist())
-    bv = sorted(b.values.tolist())
-    assert av == bv, (av, bv)
+    if isinstance(a, pd.DataFrame):
+        av = a.sort_values(list(a.columns)).values
+        bv = b.sort_values(list(b.columns)).values
+    else:
+        av = a.sort_values().values
+        bv = b.sort_values().values
+    tm.assert_numpy_array_equal(av, bv)
 
 
 @pytest.mark.parametrize('how', ['inner', 'left', 'right', 'outer'])
-def test_hash_join(how):
+@pytest.mark.parametrize('shuffle', ['disk', 'tasks'])
+def test_hash_join(how, shuffle):
     A = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': [1, 1, 2, 2, 3, 4]})
     a = dd.repartition(A, [0, 4, 5])
 
@@ -151,7 +219,7 @@ def test_hash_join(how):
     list_eq(result, expected)
 
     # Different columns and npartitions
-    c = hash_join(a, 'x', b, 'z', 'outer', npartitions=3)
+    c = hash_join(a, 'x', b, 'z', 'outer', npartitions=3, shuffle=shuffle)
     assert c.npartitions == 3
 
     result = c.compute()
@@ -159,10 +227,10 @@ def test_hash_join(how):
 
     list_eq(result, expected)
 
-    assert hash_join(a, 'y', b, 'y', 'inner')._name == \
-           hash_join(a, 'y', b, 'y', 'inner')._name
-    assert hash_join(a, 'y', b, 'y', 'inner')._name != \
-           hash_join(a, 'y', b, 'y', 'outer')._name
+    assert hash_join(a, 'y', b, 'y', 'inner', shuffle=shuffle)._name == \
+           hash_join(a, 'y', b, 'y', 'inner', shuffle=shuffle)._name
+    assert hash_join(a, 'y', b, 'y', 'inner', shuffle=shuffle)._name != \
+           hash_join(a, 'y', b, 'y', 'outer', shuffle=shuffle)._name
 
 
 @pytest.mark.parametrize('join', ['inner', 'outer'])
@@ -221,43 +289,46 @@ def test_concat(join):
 
 
 @pytest.mark.parametrize('how', ['inner', 'outer', 'left', 'right'])
-def test_merge(how):
+@pytest.mark.parametrize('shuffle', ['disk', 'tasks'])
+def test_merge(how, shuffle):
     A = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': [1, 1, 2, 2, 3, 4]})
     a = dd.repartition(A, [0, 4, 5])
 
     B = pd.DataFrame({'y': [1, 3, 4, 4, 5, 6], 'z': [6, 5, 4, 3, 2, 1]})
     b = dd.repartition(B, [0, 2, 5])
 
-    eq(dd.merge(a, b, left_index=True, right_index=True),
-       pd.merge(A, B, left_index=True, right_index=True))
+    eq(dd.merge(a, b, left_index=True, right_index=True, how=how, shuffle=shuffle),
+       pd.merge(A, B, left_index=True, right_index=True, how=how))
 
     result = dd.merge(a, b, on='y', how=how)
     list_eq(result, pd.merge(A, B, on='y', how=how))
     assert all(d is None for d in result.divisions)
 
-    list_eq(dd.merge(a, b, left_on='x', right_on='z', how=how),
+    list_eq(dd.merge(a, b, left_on='x', right_on='z', how=how, shuffle=shuffle),
             pd.merge(A, B, left_on='x', right_on='z', how=how))
     list_eq(dd.merge(a, b, left_on='x', right_on='z', how=how,
-                     suffixes=('1', '2')),
+                     suffixes=('1', '2'), shuffle=shuffle),
             pd.merge(A, B, left_on='x', right_on='z', how=how,
                      suffixes=('1', '2')))
 
-    list_eq(dd.merge(a, b, how=how), pd.merge(A, B, how=how))
-    list_eq(dd.merge(a, B, how=how), pd.merge(A, B, how=how))
-    list_eq(dd.merge(A, b, how=how), pd.merge(A, B, how=how))
-    list_eq(dd.merge(A, B, how=how), pd.merge(A, B, how=how))
+    list_eq(dd.merge(a, b, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
+    list_eq(dd.merge(a, B, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
+    list_eq(dd.merge(A, b, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
+    list_eq(dd.merge(A, B, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
 
-    list_eq(dd.merge(a, b, left_index=True, right_index=True, how=how),
+    list_eq(dd.merge(a, b, left_index=True, right_index=True, how=how,
+                     shuffle=shuffle),
             pd.merge(A, B, left_index=True, right_index=True, how=how))
     list_eq(dd.merge(a, b, left_index=True, right_index=True, how=how,
-                     suffixes=('1', '2')),
+                     suffixes=('1', '2'), shuffle=shuffle),
             pd.merge(A, B, left_index=True, right_index=True, how=how,
                      suffixes=('1', '2')))
 
-    list_eq(dd.merge(a, b, left_on='x', right_index=True, how=how),
+    list_eq(dd.merge(a, b, left_on='x', right_index=True, how=how,
+                     shuffle=shuffle),
             pd.merge(A, B, left_on='x', right_index=True, how=how))
     list_eq(dd.merge(a, b, left_on='x', right_index=True, how=how,
-                     suffixes=('1', '2')),
+                     suffixes=('1', '2'), shuffle=shuffle),
             pd.merge(A, B, left_on='x', right_index=True, how=how,
                      suffixes=('1', '2')))
 
@@ -266,9 +337,23 @@ def test_merge(how):
     #         pd.merge(A, B, left_index=True, right_on='y'))
 
 
+def test_merge_tasks_passes_through():
+    a = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
+                      'b': [7, 6, 5, 4, 3, 2, 1]})
+    b = pd.DataFrame({'c': [1, 2, 3, 4, 5, 6, 7],
+                      'd': [7, 6, 5, 4, 3, 2, 1]})
 
+    aa = dd.from_pandas(a, npartitions=3)
+    bb = dd.from_pandas(b, npartitions=2)
+
+    cc = aa.merge(bb, left_on='a', right_on='d', shuffle='tasks')
+
+    assert not any('partd' in k[0] for k in cc.dask)
+
+
+@pytest.mark.parametrize('shuffle', ['disk', 'tasks'])
 @pytest.mark.parametrize('how', ['inner', 'outer', 'left', 'right'])
-def test_merge_by_index_patterns(how):
+def test_merge_by_index_patterns(how, shuffle):
 
     pdf1l = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
                           'b': [7, 6, 5, 4, 3, 2, 1]})
@@ -318,47 +403,78 @@ def test_merge_by_index_patterns(how):
                      (pdf4l, pdf4r), (pdf5l, pdf5r), (pdf6l, pdf6r),
                      (pdf7l, pdf7r)]:
 
-        for lpart, rpart in [(2, 2),  # same partition
-                             (3, 2),  # left npartition > right npartition
-                             (2, 3)]: # left npartition < right npartition
+        for lpart, rpart in [(2, 2),    # same partition
+                             (3, 2),    # left npartition > right npartition
+                             (2, 3)]:   # left npartition < right npartition
 
             ddl = dd.from_pandas(pdl, lpart)
             ddr = dd.from_pandas(pdr, rpart)
 
-            eq(dd.merge(ddl, ddr, how=how, left_index=True, right_index=True),
+            eq(dd.merge(ddl, ddr, how=how, left_index=True, right_index=True,
+                        shuffle=shuffle),
                pd.merge(pdl, pdr, how=how, left_index=True, right_index=True))
-            eq(dd.merge(ddr, ddl, how=how, left_index=True, right_index=True),
+            eq(dd.merge(ddr, ddl, how=how, left_index=True, right_index=True,
+                        shuffle=shuffle),
                pd.merge(pdr, pdl, how=how, left_index=True, right_index=True))
 
-            eq(ddr.merge(ddl, how=how, left_index=True, right_index=True),
+            eq(dd.merge(ddl, ddr, how=how, left_index=True, right_index=True,
+                        shuffle=shuffle, indicator=True),
+               pd.merge(pdl, pdr, how=how, left_index=True, right_index=True,
+                        indicator=True))
+            eq(dd.merge(ddr, ddl, how=how, left_index=True, right_index=True,
+                        shuffle=shuffle, indicator=True),
+               pd.merge(pdr, pdl, how=how, left_index=True, right_index=True,
+                        indicator=True))
+
+            eq(ddr.merge(ddl, how=how, left_index=True, right_index=True,
+                         shuffle=shuffle),
                pdr.merge(pdl, how=how, left_index=True, right_index=True))
-            eq(ddl.merge(ddr, how=how, left_index=True, right_index=True),
+            eq(ddl.merge(ddr, how=how, left_index=True, right_index=True,
+                         shuffle=shuffle),
                pdl.merge(pdr, how=how, left_index=True, right_index=True))
 
             # hash join
-            list_eq(dd.merge(ddl, ddr, how=how, left_on='a', right_on='c'),
+            list_eq(dd.merge(ddl, ddr, how=how, left_on='a', right_on='c',
+                             shuffle=shuffle),
                     pd.merge(pdl, pdr, how=how, left_on='a', right_on='c'))
-            list_eq(dd.merge(ddl, ddr, how=how, left_on='b', right_on='d'),
+            list_eq(dd.merge(ddl, ddr, how=how, left_on='b', right_on='d',
+                             shuffle=shuffle),
                     pd.merge(pdl, pdr, how=how, left_on='b', right_on='d'))
 
-            list_eq(dd.merge(ddr, ddl, how=how, left_on='c', right_on='a'),
+            list_eq(dd.merge(ddr, ddl, how=how, left_on='c', right_on='a',
+                             shuffle=shuffle, indicator=True),
+                    pd.merge(pdr, pdl, how=how, left_on='c', right_on='a',
+                             indicator=True))
+            list_eq(dd.merge(ddr, ddl, how=how, left_on='d', right_on='b',
+                             shuffle=shuffle, indicator=True),
+                    pd.merge(pdr, pdl, how=how, left_on='d', right_on='b',
+                             indicator=True))
+
+            list_eq(dd.merge(ddr, ddl, how=how, left_on='c', right_on='a',
+                             shuffle=shuffle),
                     pd.merge(pdr, pdl, how=how, left_on='c', right_on='a'))
-            list_eq(dd.merge(ddr, ddl, how=how, left_on='d', right_on='b'),
+            list_eq(dd.merge(ddr, ddl, how=how, left_on='d', right_on='b',
+                             shuffle=shuffle),
                     pd.merge(pdr, pdl, how=how, left_on='d', right_on='b'))
 
-            list_eq(ddl.merge(ddr, how=how, left_on='a', right_on='c'),
+            list_eq(ddl.merge(ddr, how=how, left_on='a', right_on='c',
+                              shuffle=shuffle),
                     pdl.merge(pdr, how=how, left_on='a', right_on='c'))
-            list_eq(ddl.merge(ddr, how=how, left_on='b', right_on='d'),
+            list_eq(ddl.merge(ddr, how=how, left_on='b', right_on='d',
+                              shuffle=shuffle),
                     pdl.merge(pdr, how=how, left_on='b', right_on='d'))
 
-            list_eq(ddr.merge(ddl, how=how, left_on='c', right_on='a'),
+            list_eq(ddr.merge(ddl, how=how, left_on='c', right_on='a',
+                              shuffle=shuffle),
                     pdr.merge(pdl, how=how, left_on='c', right_on='a'))
-            list_eq(ddr.merge(ddl, how=how, left_on='d', right_on='b'),
+            list_eq(ddr.merge(ddl, how=how, left_on='d', right_on='b',
+                              shuffle=shuffle),
                     pdr.merge(pdl, how=how, left_on='d', right_on='b'))
 
 
 @pytest.mark.parametrize('how', ['inner', 'outer', 'left', 'right'])
-def test_join_by_index_patterns(how):
+@pytest.mark.parametrize('shuffle', ['disk', 'tasks'])
+def test_join_by_index_patterns(how, shuffle):
 
     # Similar test cases as test_merge_by_index_patterns,
     # but columns / index for join have same dtype
@@ -418,12 +534,12 @@ def test_join_by_index_patterns(how):
             ddl = dd.from_pandas(pdl, lpart)
             ddr = dd.from_pandas(pdr, rpart)
 
-            eq(ddl.join(ddr, how=how), pdl.join(pdr, how=how))
-            eq(ddr.join(ddl, how=how), pdr.join(pdl, how=how))
+            eq(ddl.join(ddr, how=how, shuffle=shuffle), pdl.join(pdr, how=how))
+            eq(ddr.join(ddl, how=how, shuffle=shuffle), pdr.join(pdl, how=how))
 
-            eq(ddl.join(ddr, how=how, lsuffix='l', rsuffix='r'),
+            eq(ddl.join(ddr, how=how, lsuffix='l', rsuffix='r', shuffle=shuffle),
                pdl.join(pdr, how=how, lsuffix='l', rsuffix='r'))
-            eq(ddr.join(ddl, how=how, lsuffix='l', rsuffix='r'),
+            eq(ddr.join(ddl, how=how, lsuffix='l', rsuffix='r', shuffle=shuffle),
                pdr.join(pdl, how=how, lsuffix='l', rsuffix='r'))
 
             """
@@ -450,8 +566,8 @@ def test_join_by_index_patterns(how):
 
 
 @pytest.mark.parametrize('how', ['inner', 'outer', 'left', 'right'])
-def test_merge_by_multiple_columns(how):
-
+@pytest.mark.parametrize('shuffle', ['disk', 'tasks'])
+def test_merge_by_multiple_columns(how, shuffle):
     pdf1l = pd.DataFrame({'a': list('abcdefghij'),
                           'b': list('abcdefghij'),
                           'c': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]},
@@ -486,26 +602,33 @@ def test_merge_by_multiple_columns(how):
             ddl = dd.from_pandas(pdl, lpart)
             ddr = dd.from_pandas(pdr, rpart)
 
-            eq(ddl.join(ddr, how=how), pdl.join(pdr, how=how))
-            eq(ddr.join(ddl, how=how), pdr.join(pdl, how=how))
+            eq(ddl.join(ddr, how=how, shuffle=shuffle), pdl.join(pdr, how=how))
+            eq(ddr.join(ddl, how=how, shuffle=shuffle), pdr.join(pdl, how=how))
 
-            eq(dd.merge(ddl, ddr, how=how, left_index=True, right_index=True),
+            eq(dd.merge(ddl, ddr, how=how, left_index=True, right_index=True,
+                        shuffle=shuffle),
                pd.merge(pdl, pdr, how=how, left_index=True, right_index=True))
-            eq(dd.merge(ddr, ddl, how=how, left_index=True, right_index=True),
+            eq(dd.merge(ddr, ddl, how=how, left_index=True, right_index=True,
+                        shuffle=shuffle),
                pd.merge(pdr, pdl, how=how, left_index=True, right_index=True))
 
             # hash join
-            list_eq(dd.merge(ddl, ddr, how=how, left_on='a', right_on='d'),
+            list_eq(dd.merge(ddl, ddr, how=how, left_on='a', right_on='d',
+                             shuffle=shuffle),
                     pd.merge(pdl, pdr, how=how, left_on='a', right_on='d'))
-            list_eq(dd.merge(ddl, ddr, how=how, left_on='b', right_on='e'),
+            list_eq(dd.merge(ddl, ddr, how=how, left_on='b', right_on='e',
+                             shuffle=shuffle),
                     pd.merge(pdl, pdr, how=how, left_on='b', right_on='e'))
 
-            list_eq(dd.merge(ddr, ddl, how=how, left_on='d', right_on='a'),
+            list_eq(dd.merge(ddr, ddl, how=how, left_on='d', right_on='a',
+                             shuffle=shuffle),
                     pd.merge(pdr, pdl, how=how, left_on='d', right_on='a'))
-            list_eq(dd.merge(ddr, ddl, how=how, left_on='e', right_on='b'),
+            list_eq(dd.merge(ddr, ddl, how=how, left_on='e', right_on='b',
+                             shuffle=shuffle),
                     pd.merge(pdr, pdl, how=how, left_on='e', right_on='b'))
 
-            list_eq(dd.merge(ddl, ddr, how=how, left_on=['a', 'b'], right_on=['d', 'e']),
+            list_eq(dd.merge(ddl, ddr, how=how, left_on=['a', 'b'],
+                             right_on=['d', 'e'], shuffle=shuffle),
                     pd.merge(pdl, pdr, how=how, left_on=['a', 'b'], right_on=['d', 'e']))
 
 
@@ -558,17 +681,85 @@ def test_cheap_single_partition_merge():
             a.merge(b, on='x', how='inner'))
 
 
-@pytest.mark.parametrize('lhs', [pd.DataFrame({'A': [1, 2, 3],
-                                               'B': list('abc'),
-                                               'C': 'foo',
-                                               'D': 1.0},
-                                              columns=list('DCBA'))])
-@pytest.mark.parametrize('rhs', [pd.DataFrame({'G': [4, 5],
-                                               'H': 6.0,
-                                               'I': 'bar',
-                                               'B': list('ab')},
-                                              columns=list('GHIB'))])
-def test_merge_maintains_columns(lhs, rhs):
+def test_merge_maintains_columns():
+    lhs = pd.DataFrame({'A': [1, 2, 3],
+                        'B': list('abc'),
+                        'C': 'foo',
+                        'D': 1.0},
+                        columns=list('DCBA'))
+    rhs = pd.DataFrame({'G': [4, 5],
+                        'H': 6.0,
+                        'I': 'bar',
+                        'B': list('ab')},
+                        columns=list('GHIB'))
     ddf = dd.from_pandas(lhs, npartitions=1)
     merged = dd.merge(ddf, rhs, on='B').compute()
     assert tuple(merged.columns) == ('D', 'C', 'B', 'A', 'G', 'H', 'I')
+
+
+@pytest.mark.parametrize('shuffle', ['disk', 'tasks'])
+def test_merge_index_without_divisions(shuffle):
+    a = pd.DataFrame({'x': [1, 2, 3, 4, 5]}, index=[1, 2, 3, 4, 5])
+    b = pd.DataFrame({'y': [1, 2, 3, 4, 5]}, index=[5, 4, 3, 2, 1])
+
+    aa = dd.from_pandas(a, npartitions=3, sort=False)
+    bb = dd.from_pandas(b, npartitions=2)
+
+    eq(aa.join(bb, how='inner', shuffle=shuffle),
+       a.join(b, how='inner'))
+
+
+def test_half_indexed_dataframe_avoids_shuffle():
+    a = pd.DataFrame({'x': np.random.randint(100, size=1000)})
+    b = pd.DataFrame({'y': np.random.randint(100, size=100)},
+                     index=np.random.randint(100, size=100))
+
+    aa = dd.from_pandas(a, npartitions=100)
+    bb = dd.from_pandas(b, npartitions=2)
+
+    c = pd.merge(a, b, left_index=True, right_on='y')
+    cc = dd.merge(aa, bb, left_index=True, right_on='y', shuffle='tasks')
+
+    list_eq(c, cc)
+
+    assert len(cc.dask) < 500
+
+
+def test_errors_for_merge_on_frame_columns():
+    a = pd.DataFrame({'x': [1, 2, 3, 4, 5]}, index=[1, 2, 3, 4, 5])
+    b = pd.DataFrame({'y': [1, 2, 3, 4, 5]}, index=[5, 4, 3, 2, 1])
+
+    aa = dd.from_pandas(a, npartitions=3, sort=False)
+    bb = dd.from_pandas(b, npartitions=2)
+
+    with pytest.raises(NotImplementedError):
+        dd.merge(aa, bb, left_on='x', right_on=bb.y)
+
+    with pytest.raises(NotImplementedError):
+        dd.merge(aa, bb, left_on=aa.x, right_on=bb.y)
+
+
+def test_concat_unknown_divisions():
+    a = pd.Series([1, 2, 3, 4])
+    b = pd.Series([4, 3, 2, 1])
+    aa = dd.from_pandas(a, npartitions=2, sort=False)
+    bb = dd.from_pandas(b, npartitions=2, sort=False)
+
+    assert not aa.known_divisions
+
+    assert eq(pd.concat([a, b], axis=1),
+              dd.concat([aa, bb], axis=1))
+
+    cc = dd.from_pandas(b, npartitions=1, sort=False)
+    with pytest.raises(ValueError):
+        dd.concat([aa, cc], axis=1)
+
+
+def test_concat_unknown_divisions_errors():
+    a = pd.Series([1, 2, 3, 4, 5, 6])
+    b = pd.Series([4, 3, 2, 1])
+    aa = dd.from_pandas(a, npartitions=2, sort=False)
+    bb = dd.from_pandas(b, npartitions=2, sort=False)
+
+    with pytest.raises(ValueError):
+        dd.concat([aa, bb], axis=1).compute()
