@@ -36,12 +36,12 @@ def _resample_series(series, start, end, reindex_closed, rule,
 
 
 def _resample_bin_and_out_divs(divisions, rule, closed='left', label='left'):
-    rule = pd.datetools.to_offset(rule)
+    rule = pd.tseries.frequencies.to_offset(rule)
     g = pd.TimeGrouper(rule, how='count', closed=closed, label=label)
 
     # Determine bins to apply `how` to. Disregard labeling scheme.
     divs = pd.Series(range(len(divisions)), index=divisions)
-    temp = divs.resample(rule, how='count', closed=closed, label='left')
+    temp = divs.resample(rule, closed=closed, label='left').count()
     tempdivs = temp.loc[temp > 0].index
 
     # Cleanup closed == 'right' and label == 'right'
@@ -77,9 +77,14 @@ def _resample_bin_and_out_divs(divisions, rule, closed='left', label='left'):
 
 class Resampler(object):
     def __init__(self, obj, rule, **kwargs):
+        if not obj.known_divisions:
+            msg = ("Can only resample dataframes with known divisions\n"
+                   "See dask.pydata.io/en/latest/dataframe-partitions.html\n"
+                   "for more information.")
+            raise ValueError(msg)
         self.obj = obj
-        rule = pd.datetools.to_offset(rule)
-        day_nanos = pd.datetools.Day().nanos
+        rule = pd.tseries.frequencies.to_offset(rule)
+        day_nanos = pd.tseries.frequencies.Day().nanos
 
         if getnanos(rule) and day_nanos % rule.nanos:
             raise NotImplementedError('Resampling frequency %s that does'
@@ -88,7 +93,7 @@ class Resampler(object):
         self._rule = rule
         self._kwargs = kwargs
 
-    def _agg(self, how, columns=None, fill_value=np.nan):
+    def _agg(self, how, meta=None, fill_value=np.nan):
         rule = self._rule
         kwargs = self._kwargs
         name = 'resample-' + tokenize(self.obj, rule, kwargs, how)
@@ -103,13 +108,18 @@ class Resampler(object):
         keys = partitioned._keys()
         dsk = partitioned.dask
 
-        args = zip(keys, outdivs, outdivs[1:], ['left']*(len(keys)-1) + [None])
+        args = zip(keys, outdivs, outdivs[1:], ['left'] * (len(keys) - 1) + [None])
         for i, (k, s, e, c) in enumerate(args):
             dsk[(name, i)] = (_resample_series, k, s, e, c,
                               rule, kwargs, how, fill_value)
-        if columns:
-            return DataFrame(dsk, name, columns, outdivs)
-        return Series(dsk, name, self.obj.name, outdivs)
+
+        # Infer output metadata
+        meta_r = self.obj._meta_nonempty.resample(self._rule, **self._kwargs)
+        meta = getattr(meta_r, how)()
+
+        if isinstance(meta, pd.DataFrame):
+            return DataFrame(dsk, name, meta, outdivs)
+        return Series(dsk, name, meta, outdivs)
 
     def count(self):
         return self._agg('count', fill_value=0)
@@ -133,7 +143,7 @@ class Resampler(object):
         return self._agg('max')
 
     def ohlc(self):
-        return self._agg('ohlc', columns=['open', 'high', 'low', 'close'])
+        return self._agg('ohlc')
 
     def prod(self):
         return self._agg('prod')
