@@ -56,9 +56,8 @@ the descendent on whose result the most tasks depend.
 """
 from __future__ import absolute_import, division, print_function
 
-from operator import add
-
-from .core import get_dependencies, reverse_dict, get_deps
+from .core import get_dependencies, reverse_dict, get_deps  # noqa: F401
+from .utils_test import add, inc  # noqa: F401
 
 
 def order(dsk, dependencies=None):
@@ -80,7 +79,11 @@ def order(dsk, dependencies=None):
 
     ndeps = ndependents(dependencies, dependents)
     maxes = child_max(dependencies, dependents, ndeps)
-    return dfs(dependencies, dependents, key=maxes.get)
+
+    def key(x):
+        return -maxes.get(x, 0), str(x)
+
+    return dfs(dependencies, dependents, key=key)
 
 
 def ndependents(dependencies, dependents):
@@ -166,13 +169,13 @@ def dfs(dependencies, dependents, key=lambda x: x):
     >>> dependencies, dependents = get_deps(dsk)
 
     >>> sorted(dfs(dependencies, dependents).items())
-    [('a', 2), ('b', 3), ('c', 1), ('d', 0)]
+    [('a', 3), ('b', 1), ('c', 2), ('d', 0)]
     """
     result = dict()
     i = 0
 
     roots = [k for k, v in dependents.items() if not v]
-    stack = sorted(roots, key=key)
+    stack = sorted(roots, key=key, reverse=True)
     seen = set()
 
     while stack:
@@ -185,12 +188,8 @@ def dfs(dependencies, dependents, key=lambda x: x):
         deps = dependencies[item]
         if deps:
             deps = deps - seen
-            deps = sorted(deps, key=key)
+            deps = sorted(deps, key=key, reverse=True)
             stack.extend(deps)
         i += 1
 
     return result
-
-
-def inc(x):
-    return x + 1
