@@ -5,21 +5,29 @@ See async.py
 """
 from __future__ import absolute_import, division, print_function
 
+from collections import defaultdict
 from multiprocessing.pool import ThreadPool
-from threading import current_thread
+import threading
+from threading import current_thread, Lock
+
 from .async import get_async
 from .compatibility import Queue
 from .context import _globals
 from .utils_test import inc, add  # noqa: F401
 
 
-default_pool = ThreadPool()
-
-
 def _thread_get_id():
     return current_thread().ident
 
 
+default_pool = ThreadPool()
+default_thread = _thread_get_id()
+main_thread = current_thread()
+
+pools = defaultdict(dict)
+pools_lock = Lock()
+
+
 def get(dsk, result, cache=None, num_workers=None, **kwargs):
     """ Threaded cached implementation of dask.get
 
@@ -45,16 +53,30 @@ def get(dsk, result, cache=None, num_workers=None, **kwargs):
     (4, 2)
     """
     pool = _globals['pool']
+    thread = current_thread()
 
-    if pool is None:
-        if num_workers:
-            pool = ThreadPool(num_workers)
-        else:
-            pool = default_pool
+    with pools_lock:
+        if pool is None:
+            if num_workers is None and thread is main_thread:
+                pool = default_pool
+            elif thread in pools and num_workers in pools[thread]:
+                pool = pools[thread][num_workers]
+            else:
+                pool = ThreadPool(num_workers)
+                pools[thread][num_workers] = pool
 
     queue = Queue()
     results = get_async(pool.apply_async, len(pool._pool), dsk, result,
                         cache=cache, queue=queue, get_id=_thread_get_id,
                         **kwargs)
 
+    # Cleanup pools associated to dead threads
+    with pools_lock:
+        active_threads = set(threading.enumerate())
+        if thread is not main_thread:
+            for t in list(pools):
+                if t not in active_threads:
+                    for p in pools.pop(t).values():
+                        p.close()
+
     return results
